;
; $Id: gui.scm,v 1.40 2003/06/20 06:57:28 jbednar Exp $
;
; The main GUI implementation.  All lissom-specific graphical code goes here.

(module gui mzscheme
   (require "ppm.scm"
	    "lissom.scm"
            "properties-panel.scm"
	    (lib "mred.ss" "mred")
            (lib "list.ss" "mzlib")
	    (lib "class.ss" "mzlib"))

   (provide (all-defined))

   (define lissom-console%
     (class frame%	    
       ; a lissom subprocess manager for talking to lissom
       ; values are initialized in (init-lissom path)
       (init-field lissom-path)
       (field (lissom-exe #f))
       (field (lissom-dir #f))
       (field (root-prefix #f))
       (field (initial-params-file #f))
       (field (lissom #f))

       ; Support for auto-refresh
       (field (panels-to-refresh '()))

       (define (add-panel-to-refresh-list panel)
	 (set! panels-to-refresh (cons panel panels-to-refresh)))

       (define/public (auto-refresh-plots)
	 (map (lambda (w) (send w auto-regenerate)) panels-to-refresh))


       (set! lissom-path (simplify-path lissom-path))
       
       (define/public (get-lissom) lissom)

       ; need to kill lissom when we close
       (override on-close)

       (define (on-close)
         (send lissom quit)
         (exit))

       ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
       ; initialize lissom
       (define init-cmds
         (list ;"call measure_or_pref"
	     
          ; set file names
          "PlotGroup::Activity::filename_format=gui.$${current_region}_$${current_plot}"
          "PlotGroup::Weights::filename_format=gui.$${current_region}_$${current_plot}"
          "PlotGroup::*Preference::filename_format=gui.$${current_region}_$${current_plot}"
          "PlotGroup::WeightsMap::filename_format=gui.$${current_region}_$${current_plot}"
                    

          ; Set parameters to generate individual images instead of combined plot
          "ppm_border=0"
          "spawn_viewer=False"
          "cmd::ppm_separate_plots=True"
          "cmd::ppm_combined_plots=False"


          )
         )

       (define (lissom-exe-name)
	 (case (system-type)
	   ((windows) "lissom.exe")
	   (else "lissom")))

       (define (init-lissom)
         (cond ((directory-exists? lissom-path)
                (set! lissom-dir lissom-path)
                (set! lissom-exe (lissom-exe-name))
                (unless (file-exists? (build-path lissom-dir lissom-exe))
                  (error "Can't find lissom executable in " lissom-dir))
                (let-values (((base name flag) (split-path lissom-path)))
                  (set! root-prefix name)))
               ((file-exists? lissom-path)
                (let ((dummy #f))
                  (set!-values (lissom-dir lissom-exe dummy) (split-path lissom-path))
                  (let-values (((base name flag) (split-path lissom-dir)))
                    (set! root-prefix name))))
               (else
                (error (format "No such file or directory ~S" lissom-path))))
         
         (printf "lissom-path = ~S~%lissom-dir = ~S~%lissom-exe = ~S~%root-prefix = ~S~%"
                 lissom-path lissom-dir lissom-exe root-prefix)
         (current-directory lissom-dir)
         (clean-up-current-directory)
         (set! lissom-dir (current-directory))
         (printf "current-directory = ~S~%" (current-directory))
         (set! lissom (instantiate lissom% () (path lissom-exe)))
         (set! initial-params-file (format "~A.param" root-prefix))
       
         (if (file-exists? initial-params-file)
             (begin
               (printf "lissomgui: loading params file ~S~%" initial-params-file)
               (send lissom cmd (format "exec_file ~A" initial-params-file))
               (send lissom cmd (format "set filebase=~A" root-prefix))
               )
		
             (printf "lissomgui: No such file ~S, not loaded.~%" initial-params-file))

	 ;; init-cmds are processed here to make sure they override anything
	 ;; that may be in the initial-params-file
         (send lissom cmds init-cmds))
       
       (define (clean-up-current-directory)
         ; delete all the old gui files
         (for-each delete-file (files-matching-regexp "^gui[.].*[.]ppm$")))
              
       (super-instantiate ("Lissom Console")
         (stretchable-height #f)
         (stretchable-width #f))

       (init-lissom)
       (current-directory lissom-dir)
   
       ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
       ; the button panel
       (field (button-panel
               (instantiate vertical-panel% () (parent this))))
       
       
       
       ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
       ; control button panel
       (field (control-button-panel 
               (instantiate horizontal-panel% () (parent button-panel))))
       
       ; Quit button
       (instantiate button% ()
         (parent control-button-panel)
         (label "Quit")
         (callback (lambda (b e) (on-close))))
       
       ; reset network button
       (instantiate button% ()
         (parent control-button-panel)
         (label "Reset network")
         (callback 
          (lambda (button event)            
            (set! lissom-path (get-simulation-dir this))
            (send lissom quit)
            (init-lissom))))
       

       ; load saved network button
       (instantiate button% ()
         (parent control-button-panel)
         (label "Reload saved network")
         (callback 
          (lambda (button event)
            (send lissom cmd  "load_snapshot"))))
       
       
       ; test pattern button
       (instantiate button% ()
         (parent control-button-panel)
         (label "Test Pattern")
         (callback (lambda (button event) 
                     (send retinal-input-params-frame show #t)) ))
       
       ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
       ; plot button panel
       (field (plot-button-panel 
               (instantiate horizontal-panel% () (parent button-panel))))
       
       (instantiate message% () (parent plot-button-panel) (label "Plots:"))
       ;;;;;;
       ; Activity button
       (instantiate button% ()
         (parent plot-button-panel)
         (label "Activity")
         (callback (lambda (button event) 
                     (send (new-activity-window) show #t))))

       ; Orientation button
       (instantiate button% ()
         (parent plot-button-panel)
         (label "Orientation Map")
         (callback (lambda (button event) 
                     (send (new-orientation-window) show #t))))
       
       
       ; Weights button
       (instantiate button% ()
         (parent plot-button-panel)
         (label "Weights")
         (callback (lambda (button event) 
                     (send (new-weights-window) show #t))))
       
       
       
       ; Units button
       (instantiate button% ()
         (parent plot-button-panel)
         (label "Weights Array")
         (callback (lambda (button event) 
                     (send (new-weights-array-window) show #t))))
       



       ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
       ; The command input panel
       (define (send-lissom-text-cmd)
         (send lissom cmd (send cmd-text get-value)))
       
       (field (cmd-panel (instantiate horizontal-panel% () 
                           (parent this))))
       (field (cmd-text (instantiate text-field%  ()
                          (callback (lambda (t e) #f))
                          (parent cmd-panel) 
                          (label "Command:"))))
       
       (field (train-panel (instantiate horizontal-panel% () (parent this))))
       (field (train-iterations (instantiate text-field% ()
                                  (init-value "1")
                                  (callback (lambda (t e) #f))
                                  (parent train-panel)
                                  (label "Iterations:"))))
       (instantiate button% ()
         (parent train-panel)
         (label "Train")
         (callback 
          (lambda (button event)
            (begin-busy-cursor)			    
            (send lissom cmd  
                  (format "training +~A"
                          (send train-iterations get-value)))
	    (auto-refresh-plots)
            (end-busy-cursor))))

       (instantiate button% ()
         (parent cmd-panel)
         (label "Send")
         (style '(border))
         (callback (lambda (button event) 
                     (begin-busy-cursor)
                     ;(send cmd-text enable #f)
                     (send-lissom-text-cmd)
                     ;(send cmd-text enable #t)
                     (end-busy-cursor)
                     )
                   ))
				 
       


       ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
       ; activity windows
       
       
       (field (activity-window-count 0))
       
       (define (new-activity-window)
         (set! activity-window-count (+ activity-window-count 1))
         (let* ((frame (instantiate frame% 
                         ((format "Activity ~A" activity-window-count))
                         (stretchable-height #f)
                         (stretchable-width #f)))
                (panel (instantiate plot-panel% ()
                         (parent frame)
                         (console this))))
           (send panel regenerate)
	   (add-panel-to-refresh-list panel)
	   (send panel set-auto-refresh #t) ; Default to on for Activity windows
           frame))

			 

       ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
       ;  weights windows
       
       (field (weights-window-count 0))
       (define (new-weights-window)
         (set! weights-window-count (+ weights-window-count 1))
         (let* ((frame (instantiate frame% ((format "Weights ~A" weights-window-count))
                         (stretchable-height #f)
                         (stretchable-width #f)))
                (panel (instantiate weights-panel% ()
                         (parent frame)
                         (console this))))
           (send panel regenerate)
	   (add-panel-to-refresh-list panel)
           frame))
       
       
       ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
       ; orientation windows
       
       (field (orientation-plot-cmds (list "call measure_or_pref" "plot verbose=true OrientationPreference " "plot verbose=true OcularPreference ")))
       (field (orientation-window-count 0))
       (define (new-orientation-window)
         (set! orientation-window-count (+ orientation-window-count 1))
         (let* ((frame (instantiate frame% ((format "Orientation ~A" orientation-window-count))
                         (stretchable-height #f)
                         (stretchable-width #f)))
                (panel (instantiate plot-panel% ()
                         (parent frame)
                         (console this)
                         (regenerate-cmds orientation-plot-cmds))))
           (send panel regenerate)
	   (add-panel-to-refresh-list panel)
           frame))
       
       ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
       ; weights array windows
       
       (field (weights-array-window-count 0))
       
       (define (new-weights-array-window)
         (set! weights-array-window-count (+ weights-array-window-count 1))
         (let* ((frame (instantiate frame% ((format "Weights Array ~A" 
                                                    weights-array-window-count))
                         (stretchable-height #f)
                         (stretchable-width #f)))
                (panel (instantiate weights-array-panel% ()
                         (parent frame)
                         (initial-width 256)
                         (console this) )))
           (send panel regenerate)
	   (add-panel-to-refresh-list panel)
           frame))

       ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
       ; The retinal input params panel
       (field (retinal-input-params-frame
               (instantiate frame% ("Test pattern parameters")
                 (stretchable-width #f)
                 (stretchable-height #f))))
       (field (retinal-input-params-panel
               (instantiate retinal-input-params-panel% ()
                 (console this)
                 (parent retinal-input-params-frame))))
       

       ))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

  (define plot-panel%
     (class vertical-panel%
       (init-field console)
       (init-field (files ()))
       (init-field (labels ()))
       (init-field (initial-width 128))
       (init-field (regenerate-cmds (list "plot verbose=true")))
       (field (filename-format-str "gui.~A_~A.ppm"))
       ; To be set to true if the filename-format-str uses a 
       ; type parameter in addition to the region parameter
       (field (filename-uses-type #t))
       
       (field (zoom-increment 1/4))
       
       (define/public (regenerate)
         (begin-busy-cursor)
         ;execute the regenerate command
         (do-cmds regenerate-cmds)
         (end-busy-cursor))
       
       (define/public (auto-regenerate)
	 (if (send auto-refresh-control get-value) (regenerate)))
       
       (define/public (set-auto-refresh val)
	 (send auto-refresh-control set-value val))

       (define (do-cmd-with-info c)
         ;
         ; Execute a lissom command and parse the output
         ; to discover which files were plotted.
         ; Return "info" in the form of a list of plot file names, and plot labels,
         ; e.g. (("gui.Eye0_Activity.ppm" "Eye0 Activity") ("gui.Ganglia00_Activity.ppm" "Ganglia00 Activity"))
         ;
         (let* ((reply-port (send (send console get-lissom) cmd c 'reply-as-port ))
                ; get the lines of the reply
                (lines (let loop ((results ()))
                         (let ((line (read-line reply-port)))
                           (if (eof-object? line)
                               (reverse results)
                               (loop (cons line results)))))))
           ; loop through the lines extracting the region and type of each plot
           (let loop ((results ())
                      (l lines))
             (if (null? l)
                 (reverse results)
                 (let ((match (regexp-match "Plotting ([^ ]*)::([^ ]*)[ .]" (car l))))
                   (if match
                       (let* ((region (cadr match))
                              (type (caddr match))
                              (filename (if filename-uses-type
					    (format filename-format-str region type)
					    (format filename-format-str region)))
                              (label (format "~A ~A" region type)))
                         (if (file-exists? filename)
                             (loop (cons (list filename label) results) (cdr l))
                             (begin
                               (printf "No such file: ~S in ~S~%" filename (current-directory))
                               (loop results (cdr l)))))
                       (loop results (cdr l))))))))

       (define/public (do-cmds cmds)
         ;
         ; Execute the lissom commands and concatenate their plot info.
         ; Then if the list of files returned is different from the current list of
         ; plot files, rebuild the display, otherwise just reload the listed files.
         ;
         (let* ((plot-info (apply append (map (lambda (c) (do-cmd-with-info c)) cmds)))
                (new-files (map car plot-info))
                (new-labels (map cadr plot-info)))
           
           ;(printf "new-files = ~S~%new-labels = ~S~%" new-files new-labels)
           (if (not (and (equal? new-files files) (equal? new-labels labels)))
               ; if files or labels changed, re-init the plot-array
               (begin
                 ;(printf "New plot files: ~A~%" new-files)
                 (set! files new-files)
                 (send plot-array set-files files)
                 (set! labels new-labels)
                 (send plot-array set-labels labels)
                 (send plot-array reinit))               
               ; otherwise reload the array
               (send plot-array reload))))
         
         

       (super-instantiate ())

       (field (plot-array (instantiate h-ppm-array% ()
                            (parent this)
                            (initial-width initial-width)
                            (files files)
                            (labels labels))))

       (field (control-panel (instantiate horizontal-panel% ()
                               (stretchable-height #f)
                               (parent this))))
       (instantiate button% ()
         (parent control-panel)
         (label "Refresh")
         (style '(border))
         (callback (lambda (b e) (regenerate))))
       
       (instantiate button% ()
         (parent control-panel)
         (label "Reduce")
         (callback
          (lambda (b e)
            (send plot-array zoom-by (/ (+ 1 zoom-increment))))))
       
       (instantiate button% ()
         (parent control-panel)
         (label "Enlarge")
         (callback
          (lambda (b e)
            (send plot-array zoom-by (+ 1 zoom-increment)))))
                 
       (field (auto-refresh-control (instantiate check-box% ()
						 (parent control-panel)
						 (label "Auto-refresh")
						 (callback (lambda (t e) #f)))))
       ))


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

   (define weights-panel%
     (class plot-panel%
       (inherit-field control-panel)
       (inherit-field plot-array)
       (inherit-field files)
       
       (super-instantiate ())
       ;(send plot-array init)
       
       (override regenerate)
       (define (regenerate)
         (begin-busy-cursor)
         ;(printf "weights-panel files: ~S~%" files)
         (send this do-cmds
               (list 
                (format "plot_unit verbose=true region=~A ~A ~A" 
                        (get-choice region-names region-selector)
                        (send row-control get-value)
                        (send col-control get-value))))
         (end-busy-cursor))

       (field (weight-params-panel 
               (instantiate horizontal-panel% () (parent this))))

       ;; This list should be constructed automatically by asking
       ;; lissom what regions are defined (with internal=False), but
       ;; it's currently hard-coded.
       (field (region-names '("Primary" "Ganglia00" "Ganglia01" "Secondary")))
       (field (region-selector (choice-widget "Region:" weight-params-panel region-names)))

       (field (row-control (instantiate text-field% ()
					(parent weight-params-panel)
					(label "Row:")
					(init-value "current_height/2")
					(callback (lambda (t e) #f)))))
       (field (col-control (instantiate text-field% ()
					(parent weight-params-panel)
					(label "Column:")
					(init-value "current_width/2")
					(callback (lambda (t e) #f)))))))
	       
  
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
  
   (define weights-array-panel%
     (class plot-panel%
       (inherit-field control-panel)
       (inherit-field console)
       (inherit-field plot-array)
       (inherit-field filename-format-str) 
       (inherit-field filename-uses-type) 
       (super-instantiate ())

       ;; Hack: This type of plot turns off ppm_separate_plots because
       ;; otherwise there would be hundreds or thousands of images on
       ;; disk.  The result is one single .ppm file on disk containing
       ;; all of the weight plots.  But a confusing side effect is
       ;; that the resulting .ppm file will be named for the set of
       ;; plots as a group, i.e. Weights, instead of the name of each
       ;; individual plot within the group (usually Afferent0 or
       ;; Afferent00, but can also be LateralExcitatory,
       ;; LateralInhibitory, etc.)  That makes it different from all
       ;; the other types of plots.  As a workaround, we hard-code the
       ;; type field in the filename to be the PlotGroup,
       ;; i.e. Weights, and make sure that when the
       ;; filename-format-str is evaluated only the region is passed
       ;; to it.  (If the type were also passed, format would raise an
       ;; exception, because there is no corresponding ~A).
       ;; 
       (set! filename-format-str "gui.~A_Weights.ppm")
       (set! filename-uses-type #f)

       (override regenerate)
       (define (regenerate)
	 (begin-busy-cursor)
	 ;; If situating, turns on borders and then turns them back off
	 ;; (For some reason ppm_border can't be supplied as an argument,
	 ;; and so as a sideeffect this code will overwrite the existing
	 ;; ppm_border value ...
	 (let ((situating (send situate-control get-value)))
	   (send this do-cmds
                 (list
                  (format "~Aplot_unit_range region=~A name=~A verbose=true ppm_separate_plots=false ppm_combined_plots=true ppm_neuron_skip_aff=~A weight_situate=~A ~A"
                          (if situating "ppm_border=1 " "")
			  (get-choice region-names region-selector)
			  (get-choice weight-names weight-selector)
                          (send skip-control get-value)
                          (if situating "true" "false")
                          (if situating "weight_bare=false ppm_outline_width=1 ppm_interior_border=0 ppm_interior_outline=False" ""))))
	   (if situating (send (send console get-lissom) cmd "set ppm_border=0")))
	 ;(send plot-array repopulate)
	 (end-busy-cursor))

       (field (weight-params-panel 
               (instantiate horizontal-panel% () (parent this))))

       (field (region-names '("Primary" "Ganglia00" "Ganglia01" "Secondary")))
       (field (region-selector (choice-widget "Region:" weight-params-panel region-names)))

       ;; Should ask lissom for all of these names instead of hardcoding them;
       ;; Currently asks only for the one name that changes often (Afferent00 or Afferent0)
       (field (weight-names (list 
			     (send (send console get-lissom) eval-expr "cmd::plot_unit_range::name")
			     "Afferent0"
			     "Afferent10"
			     "LateralExcitatory"
			     "LateralInhibitory")))
       (field (weight-selector (choice-widget "Name:" weight-params-panel weight-names)))

       (field (weight-params-panel2
               (instantiate horizontal-panel% () (parent this))))

       (field (skip-control (instantiate text-field% ()
					(parent weight-params-panel2)
					(label "Skip:")
					(init-value "N/8")
					(callback (lambda (t e) #f)))))
       (field (situate-control (instantiate check-box% ()
					(parent weight-params-panel2)
					(label "Situate")
					(callback (lambda (t e) #f)))))
       (send situate-control set-value #f)))
	       



  
  
  ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
  
  
  
  
  
  
   (define retinal-input-params-panel%
     (class vertical-panel%
        (public get-cmd)

	(define (get-cmd)
	  (printf "get-cmd~%")
	  (let* ((chosen-type (get-choice input-types type-selector))
		 (photo? (equal? chosen-type "Photograph"))
		 (type   (if photo? "PGM" chosen-type)))
	    (format 
	     "input_present_object learning=~A All Input_~A ~A theta=~A cx=~A cy=~A xsigma=~A ysigma=~A scale=~A offset=~A freq=~A phase=~A center_width=~A size_scale=~A"
	     (if (send learning-control get-value) "true" "false")
	     type
	     (if photo?
		 (if (equal? (send params-panel get-value 'stock-image) "(specified below)")
		     (string-append "'" (send params-panel get-value 'filename) "'")
		     (string-append "'" (send params-panel get-value 'stock-image) "'"))
		 "")
	     (send params-panel get-value 'theta)
             (send params-panel get-value 'cx) 
             (send params-panel get-value 'cy)
             (send params-panel get-value 'xsigma)
             (send params-panel get-value 'ysigma)
             (send params-panel get-value 'scale)
             (send params-panel get-value 'offset)
             (send params-panel get-value 'freq)
             (send params-panel get-value 'phase)
             (send params-panel get-value 'center_width)
             (send params-panel get-value 'size_scale)
             )))

	(define (get-training-pattern-cmd)
	  (printf "get-training-pattern-cmd~%")
	  (let* ((chosen-type (get-choice input-types type-selector))
		 (photo? (equal? chosen-type "Photograph"))
		 (type   (if photo? "PGM" chosen-type)))
	    (format 
	     "exec input_undefine 'input_define Obj0 All Input_~A ~A xsigma=~A ysigma=~A scale=~A offset=~A freq=~A phase=~A center_width=~A size_scale=~A'"
	     type
	     (if photo?
		 (if (equal? (send params-panel get-value 'stock-image)  "(specified below)")
		     (string-append "'" (send params-panel get-value 'filename) "'")
		     (string-append "'"(send params-panel get-value 'stock-image)  "'"))
		 "")
             (send params-panel get-value 'xsigma)
             (send params-panel get-value 'ysigma)
             (send params-panel get-value 'scale)
             (send params-panel get-value 'offset)
             (send params-panel get-value 'freq)
             (send params-panel get-value 'phase)
             (send params-panel get-value 'center_width)
             (send params-panel get-value 'size_scale))))


	;; This should be re-implemented using a macro to make sure that
	;; the two values match.
	(define (reset-to-defaults)
	  (send params-panel set-value  'theta  "PI/2")
	  (send params-panel set-value  'cx     "RN/2")
	  (send params-panel set-value  'cy     "RN/2")
	  (send params-panel set-value  'xsigma "7.5" )
	  (send params-panel set-value  'ysigma "1.5" )
	  (send params-panel set-value  'scale  "1.0" )
	  (send params-panel set-value  'offset "0.0" )
	  (send params-panel set-value  'freq   "0.5" )
	  (send params-panel set-value  'phase  "PI/2")
	  (send params-panel set-value  'center_width "10"  )
	  (send params-panel set-value  'size_scale   "1.0" )
	  (send params-panel set-value  'filename     ""    )
          (send params-panel set-value  'stock-image 0)
	  )

	(super-instantiate ())

	(init-field console)
	
	;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
	; control h-panel
	(field (control-panel (instantiate horizontal-panel% () (parent this))))

	;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
	; control h-panel
	(field (options-panel (instantiate horizontal-panel% () (parent this))))

	;;;;;;;;;;;;;;;;;;;;;;;;;;;;
	; input types and selector
	(field (input-types '("Gaussian"
			      "SineGrating"
			      "Gabor"
			      "UniformRandomNoise"
			      "Rectangle"
			      "FuzzyLine"
			      "FuzzyDisc"
			      "Photograph")))
	(field (type-selector (choice-widget "Input Type:" control-panel input-types)))

	;;;;;;;;;;;;;;;;;;;;;;;;;;;;
	; present button
	(instantiate button% ()
	  (parent control-panel)
	  (label "Present")
	  (style '(border))
	  (callback (lambda (b e) 
		      (send (send console get-lissom) cmd (get-cmd))
		      (send console auto-refresh-plots))))
	      

	;;;;;;;;;;;;;;;;;;;;;;;;;;;;
	; reset-to-defaults button
	(instantiate button% ()
	  (parent options-panel)
	  (label "Reset to Defaults")
	  (callback (lambda (b e) 
		      (reset-to-defaults))))
	      

	;;;;;;;;;;;;;;;;;;;;;;;;;;;;
	; use-for-training button
	(instantiate button% ()
	  (parent options-panel)
	  (label "Use for Training")
	  (callback (lambda (b e) 
		      (send (send console get-lissom) cmd (get-training-pattern-cmd)))))
	      

	;;;;;;;;;;;;;;;;;;;;;;;;;;;
	; learning checkbox
	(field (learning-control
		(instantiate check-box% () 
			     (parent options-panel)
			     (label "Learning")
			     (callback (lambda (c e) #f)))))
	
		

	; ;;;;;;;;;;;;;;;;;;;;;;;
	; params
	;
       (field (params-panel
               (instantiate properties-panel% ()
                 (parent this)
                 (stretchable-height #f))))
       
       
       ;; Somewhat safely evaluates expressions in lissom, substituting zero if 
       ;; the expression can't be parsed (as for e.g. partially typed input)
       (define (lissom-eval-expr s)
	 (let ((val (send (send console get-lissom) eval-expr s)))
	   (if val (string->number val) 0)))

       ;; Basic call to define a numeric parameter with fewer keystrokes
       (define (add-slider name min-value max-value init-value)
	 (send params-panel add-tagged-slider-property 
	       name (symbol->string name) min-value max-value init-value 3 lissom-eval-expr))

       ;;           name         min-value  max-value  init-value
       ;;				    	       
       (add-slider 'theta        "0"        "PI*2"     "0"       )
       (add-slider 'cx           "-RN/4"    "5*RN/4"   "RN/2"    )
       (add-slider 'cy           "-RN/4"    "5*RN/4"   "RN/2"    )
       (add-slider 'xsigma       "0"        "RN"       "7.5"     )
       (add-slider 'ysigma       "0"        "RN"       "7.5"     )
       (add-slider 'center_width "0"        "RN"       "10"      )
       (add-slider 'scale        "0"        "3"        "1"       )
       (add-slider 'offset       "-3"       "3"        "0"       )
       (add-slider 'freq         "0.01"     "1.25"     "0.5"     )
       (add-slider 'phase        "0"        "PI*2"     "PI/2"    )
       
       ; pre-defined images (not really useful unless easy to change or extend)
       (field (stock-images '("small/arch.pgm"
                              "small/ellen_arthur.pgm"
                              "small/gene.pgm"
                              "small/lochnessroad.pgm"
                              "small/loghog.pgm"
                              "small/skye.pgm"
                              "(specified below)"
                              )))

       (send params-panel add-choice-property 'stock-image "Photograph" stock-images 0)
       (send params-panel add-text-property 'filename "filename:" "")
       (add-slider 'size_scale   "0"     "5"       "1"    )

       (reset-to-defaults)
       ))


   ;; Saves typing for choice% widgets
   (define (choice-widget selector-label parent-panel choice-names)
     (instantiate choice% ()
		  (parent parent-panel)
		  (label selector-label)
		  (choices choice-names)
		  (callback (lambda (c e) #f))))
   
   (define (get-choice choice-names choice-selector)
     (list-ref choice-names (send choice-selector get-selection) ))


  (define (files-matching-regexp exp)
    (filter 
     (lambda (x)
       (regexp-match exp x))
     (directory-list)))

  (define (null-callback c e) #f)
  
  (define (get-simulation-dir parent)
    (get-directory "Select a simulation directory" parent (current-directory)))
  
);end of module



