;
; $Id: lissom.scm,v 1.15 2003/01/19 08:14:32 jp Exp $
;
; The lissom class: a class to manage communication with a lissom
; subprocess.  

(module lissom mzscheme

  (require (lib "class.ss" "mzlib")
           (lib "defmacro.ss" "mzlib") )
  (provide (all-defined))

  (define lissom%
    (class object%
      (field (subproc-in #f))
      (field (subproc-out #f))
      (field (subproc-err #f))
      (field (subproc #f))
      (init-field path)
      (field (prompt-string "lissomsubprocess: "))


      (field (stderr-thread #f))

      (define/public (start)

	(display (format "***** starting LISSOM executable: ~S *****~%" path) )

	(let ((prompt-arg (format "command_prompt_format=\"~A\"" prompt-string)))
	  (display prompt-arg)
	  (newline)
	  (set!-values (subproc subproc-out subproc-in subproc-err)
		       (subprocess #f #f #f path prompt-arg )))
	(set! stderr-thread (thread grab-stderr))
	(write-lissom-reply)
	(write-lissom-reply) ; hack! second call needed because prompt string is written during initial reply
        )
	
      (define/public (quit)
	(display (format "****** Quitting LISSOM *******~%"))
	(fprintf subproc-in "quit~%")
	(close-output-port subproc-in)
	(close-input-port subproc-out)
	(close-input-port subproc-err)
	(sleep 0.5)
	(kill-thread stderr-thread)
	(when (eq? (subprocess-status subproc) 'running)
	      (printf "Killing LISSOM subprocess.~%")
	      (subprocess-kill subproc)))

      (define/public (restart new-path)
        (quit)
        (set! path new-path)
        (start))
        
      
      (define/public (cmd string . opts)
	(let ((cmd-string (format "~A~%" string))
              (result #f))
	  (printf "lissom command: ~A" cmd-string)
	  (fprintf subproc-in cmd-string)
          
          (cond ((member 'reply-as-port opts)
                 (set! result (get-lissom-reply)))
                (else                 
                 (write-lissom-reply)))
	  (flush-output (current-error-port))
          result)	)

      (define/public (cmds l)
	(for-each (lambda (c) (cmd c)) l))
      
      (define/public (eval-expr expr)
        (fprintf subproc-in (format "print ~A~%" expr))
        (let ((subproc-out-lines (get-lissom-reply))
	      (reply-regexp (regexp (format " == (.*)[.]" ))))

          (let loop ((line (read-line subproc-out-lines)))	    
	    (if (eof-object? line) #f
	        (let ((matches (regexp-match reply-regexp line)))
		  (cond ((not matches)
			 (loop (read-line subproc-out-lines)))
			(else
			 (cadr matches))))))))

      (define (write-lissom-reply)
	(let ((matches (regexp-match prompt-string subproc-out 0 #f (current-output-port))))
	  ;(printf "~A" (car matches))
	  (flush-output)
	  (flush-output (current-error-port))
	  (values)))

      (define (get-lissom-reply)
        (let-values ([(reply-in-port reply-out-port) (make-pipe)])
          (regexp-match prompt-string subproc-out 0 #f reply-out-port)
          (close-output-port reply-out-port)
          reply-in-port))
      
      
      (define (grab-stderr)
	(when (and (char-ready? subproc-err)
		   (eq? (subprocess-status subproc) 'running))
	      (let ((c (read-char subproc-err)))
		(unless (eof-object? c)
			(write-char c (current-error-port))
			(grab-stderr)))))


      (super-instantiate ())
      (start)))
  

  (define-macro (lissom-cmd-button parent label lissom cmd)
    ; a macro to create a button which executes a lissom  command.
    ; Takes the parent graphical container of the button, its label, a
    ; lissom object, and a command string
    `(instantiate button% ()
       (parent ,parent)
       (label ,label)
       (callback
        (lambda (b e)
          (send ,lissom cmd ,cmd)))))
    
  (define-macro (lissom-cmds-button parent label lissom cmds)
    ; like lissom-cmd-button but takes a list of command strings
    `(instantiate button% ()
       (parent ,parent)
       (label ,label)
       (callback
        (lambda (b e)
          (send ,lissom cmds ,cmds)))))
    
  
  )