;
; $Id: tagged-slider.scm,v 1.6 2003/01/19 08:14:32 jp Exp $
;
; Code for a modified slider control that has a text-field% "tag".
; The tagged slider allows value modification through either the text
; field or the tag, with mutual updating.  If the user enters a tag
; value that exceeds the slider bounds, the bounds are extended.
; Tagged sliders can handle arbitrary real values, not just positive
; integers.
;
(module tagged-slider mzscheme
  (require (lib "mred.ss" "mred")
           (lib "class.ss" "mzlib"))
  
  (provide (all-defined))
  
  (define tagged-slider%
    (class horizontal-panel%
      (init-field min-value)
      (init-field max-value)
      (init-field (callback #f))
      (init-field (init-value min-value))
      (init-field (decimal-places 0))
      (init-field (min-tag-width 50))
      (init-field (string-translator-fn string->number))
      
      (when (string? min-value)
        (set! min-value (string-translator-fn min-value)))
      (when (string? max-value)
        (set! max-value (string-translator-fn max-value)))
      (when (string? init-value)
        (set! init-value (string-translator-fn init-value)))
      
      (super-instantiate ())
      
      
      (field (slider (instantiate slider% ()
                       (label #f)
                       (parent this)
                       (min-value 0)
                       (max-value 10000)
                       (init-value (external->internal init-value))
                       (style '(plain horizontal))
                       (callback
                        (lambda (s e)
                          (set-tag-from-slider))
                        ))))
      
      (field (tag (instantiate text-field% () 
                    (label #f)
                    (parent this)
                    (min-width min-tag-width)
                    (init-value (number->string init-value))
                    (callback
                     (lambda (t e)
                       (let ((ext-val (string-translator-fn (send t get-value))))
                         (when ext-val                         
                           (set-slider-from-tag))))
                     ))))
      
      (define (set-tag-from-slider)
        (send tag set-value (number->string (get-inexact-value))))
      
      (define (set-slider-from-tag)
        (let ((tag-value (string-translator-fn (send tag get-value))))
          (cond ((< tag-value min-value)
                 (set! min-value tag-value))
                ((< max-value tag-value)
                 (set! max-value tag-value)))
          (send slider set-value
                (external->internal (string-translator-fn (send tag get-value))))))
      
      
      (define/public (get-inexact-value)
        (round-float (exact->inexact (internal->external (send slider get-value))) decimal-places))
      
      ;; Returns string from tag, not slider value, because tag can hold higher 
      ;; precision than slider can (e.g. 2/3, 1.00005)
      (define/public (get-value) (send tag get-value))
      
      (define/public (set-value v)
        (cond ((number? v)
               (send tag set-value (number->string v)))
              ((string? v)
               (send tag set-value v))
              (else
               (error "Tagged slider value must be a number or string." v)))
        (set-slider-from-tag))
      
      
      (define/public (internal->external v)
        (let ((range (- max-value min-value)))
          (+ min-value (* (/ v 10000) range))))
      
      (define/public (external->internal v)
        (let ((range (- max-value min-value)))
          (inexact->exact (floor (* 10000 (/ (- v min-value) range))))))
      
      (define (round-float x places)
        (if (= places 0) 
            (round x)
            (let ((base (expt 10 places)))
              (/ (round (* x base)) base))))
      
      )) 
           
  )

