//
// $Id: Lissom_py.cpp,v 1.3 2004/01/11 03:24:01 jp Exp $
//

#include <iostream>
#include <ctime>
#include "Lissom_py.h"
#include "../src/cmdparam.h"
#include "../src/lissom.h"
#include "../src/globals.h"
#include "../src/neuralregionmap.h"
#include "../src/stringutils.h"

using std::cout;
using std::endl;
using std::string;
using std::vector;

void __init__(void)
{
  /* Act as if command line had been empty and command was named pythonlissom */
  int   argc=1;  /* Dummy */
  char* argv[2]; /* Dummy */
  argv[0]="pythonlissom";

  time_t start_time = time(NULL);

  cout << "Lissom __init__" << endl;

  /* Initialize various modules */
  lissom_init_hooks();

  /* Announce program start */
  ipc_notify(IPC_ONE,IPC_SUMMARY,"Execution of %s started %.24s on %s",
	     argv[0], ctime(&start_time), getenv("HOST") );

  const bool is_interactive = process_command_line_args(argc,argv);
  
  /* Read further commands from terminal prompt, if appropriate */
  if (is_interactive) {
    if (!has_command_file) 
      ipc_init_logfile(filebase.c_str());
    cmdparam_changes_verbose=True;
  }
}

void cmd(char *c)
{
  cout << "lissom cmd: " << c << endl;
  cmddefs_exec_str(c);
}

/** Parses the given expression into a double, then returns it as a string.
    Should probably change it to return the double directly, instead of the
    string. */
double eval_expr(char* expr)
{
  cout << "lissom eval_expr: " << expr << endl;
  const CmdParamStringParser sp(blackboard);
  double result;
  sp.parse(expr,result);
  return result;
}


/** Returns the value of the given parameter, as a string. */
std::string eval_param(char* name)
{
  cout << "lissom eval_param: " << name << endl;
  const ParamType& param = blackboard.lookup_with_path(name);
  /* Should we raise an exception for an invalid parameter? */
  return (param.is_valid()?  param.stringrep() : "<invalid-parameter>");
}


/** Calls a plot command (plot, plot_unit, etc.) and returns a list of
    regions that *may* have been plotted */
std::vector<string> plot_cmd(char* c)
{
  cout << "lissom plot_cmd: " << c << endl;
  cmddefs_exec_str(c);
  
  vector<string> result=NeuralRegionMap::get_current_plots();

  return result;
}

