/** \file
  Bounded namespace and associated types.

  $Header: /u/nn/cvsroot/lissom/src/bounded.h,v 1.1 2001/03/15 00:30:24 jbednar Exp $
*/

#ifndef __BOUNDED_H__
#define __BOUNDED_H__


/** Namespace for types with well-specified bounds, either enforced
  or nominal.  A design criterion is that the memory footprint of an
  instance of each class must be no larger than the size of its
  underlying type.  As a result, no virtual functions can be used,
  since the size of the virtual function table is quite large compared
  to the size of e.g. a char.  (Inheritance from a base type also
  seems to increase the class size in GCC, at least for chars, so that
  too should usually be avoided.)
*/
namespace Bounded {


/** Encompassing integral type; numeric Bounded classes can only
    accept integral values up to those representable in this type. */
typedef long LargeInt;


/** Encompassing floating-point type; numeric Bounded classes can only
    accept floating-point values up to those representable in this type. */
typedef double LargeFloat;


/** Generic type for a real-valued magnitude on the scale 0 to 1.0.
    Larger or smaller values are acceptable (boundaries are not enforced)
    but are not always meaningful. */
typedef double Magnitude;

 
} /* namespace Bounded */

#endif /* __BOUNDED_H__ */
