/** \file
  Main function for the RF-LISSOM command-line program.
  Goes with the code in lissom.c, but kept separate here so that the
  main() function can be disabled.

  $Header: /u/nn/cvsroot/lissom/src/clmain.c,v 1.1 2003/02/13 10:23:34 jbednar Exp $
*/
#include <time.h>
#include <stdio.h>

#include "ipc.h"
#include "cmdparam.h"
#include "lissom.h"
#include "globals.h"

/** Main command-line function. */
int main(int argc, char **argv)
{
  time_t start_time = time(NULL);

  /* Initialize various modules */
  lissom_init_hooks();

  /* Announce program start */
  ipc_notify(IPC_ONE,IPC_SUMMARY,"Execution of %s started %.24s on %s",
	     argv[0], ctime(&start_time), getenv("HOST") );

  /* Read and execute commands and files specified on the command line */
  const bool is_interactive = process_command_line_args(argc,argv);
  
  /* Read further commands from terminal prompt, if appropriate */
  if (is_interactive) {
    if (!has_command_file) 
      ipc_init_logfile(filebase.c_str());
    cmdparam_changes_verbose=True;
    cmddefs_exec_batch(&get_line_from_prompt,"prompt");
  }
  
  /* Exit */
  ipc_exit(IPC_EXIT_NORMAL,"Exited normally");
  return 0; /* never reached */
} 
