/** \file
  Public headers for globals.c.
  $Header: /u/nn/cvsroot/lissom/src/globals.h,v 1.63 2002/07/17 02:22:08 yschoe Exp $
*/

#ifndef __GLOBALS_H__
#define __GLOBALS_H__

#include "cmdparam.h"


/** Assume single-processor architecture if unknown */  
#ifndef NPES
#define NPES 1
#endif

/** Declare which PE is the master (the oldest PE) and
   alternate master (the youngest PE) */
#define PARENTPE     0
#define YOUNGESTPE   (NPEs-1)
#define AMPARENTPE   (MyPE==PARENTPE)
#define AMYOUNGESTPE (MyPE==YOUNGESTPE)


/* Common question to ask */
#define PEISME(pe) ((pe)==MyPE)


/**
  Marker for dead connection, usually stored as a negative value to
  distinguish it from live weights.  The value must be small
  (i.e. much less than 1.0, the total sum of weights), so that
  read-only routines can simply process values without first checking
  if they are dead.  (This hack is an optimization that saves a little
  time when computing responses, but probably not enough time to be
  worth all the confusion it causes.)

  All code that ever modifies a weight must respect dead weights,
  i.e. not modify any weight whose current value is equal to or less
  than this value.
*/
#define DEATH_FLAG        1.0E-10
//#define DEATH_FLAG        1.0E10


/* Public parameters */
extern int    iteration;
extern int    presentation;
extern int    MyPE;
extern int    network_initialized;
extern int    NPEs;
extern string filebase;


/* Public functions */
SETFN_DECLARE2(fixed_arch_param_setfn2);
SETFN_DECLARE2(read_only_param_setfn2);
void   globals_init_hook( void );
void   kernel_init_hook( void );

#endif

