/** \file
  External interface to the Eyes class.
  
  $Header: /u/nn/cvsroot/lissom/src/inputs.h,v 1.26 2001/02/22 08:17:15 jbednar Exp $
*/

#ifndef __INPUTS_H__
#define __INPUTS_H__

#include "vgen.h"

/** Class with responsibility for the entire set of network inputs. */
class Inputs : public ValueGen {
public:
  virtual ~Inputs() { }

  /** Call before any call to next or activate. */
  virtual void init()=0;
  
  /** Reset to the init state. */
  virtual void reset()=0;
  
  /** Advance contents of the inputs to the next state. */
  virtual bool next()=0;
  
  /** Compute input from current state. */
  virtual void activate(bool learn=false, bool settle=true, bool activatefn=true)=0;
  
  /** Reset to the state before init was called. */
  virtual void uninit()=0;

  /** Register user-definable parameters and commands */
  virtual void   register_params_and_commands( void ) =0;

  
  /*
    Highly deprecated interface to a few quantities sometimes needed
    by other files. Only use these when absolutely necessary, since
    the caller will have to know how many input dimensions there are,
    how objects are numbered, etc.  Without that knowledge, a caller must
    allow for the possibility that these trivial stubs are being called
    instead of something meaningful.
  */
  /** Whether the given object is currently enabled in the given eye. */
  virtual bool   is_active(int, int) const  { return false;  }
  /** The angle, in radians, of the given object in the given eye. */
  virtual double angle_of_object(int, int) const  {  return 0;  }
  /** Returns the nominal angle of the most active object at the given location in the given eye. */
  virtual double angle_of_object_at_location(int, int, int) const {  return 0;  }

  /**
     Global point of entry into a single instance of an Inputs class.  The
     actual type of this object is unknown to users.
  */
  static Inputs& global_entry();
};



#endif
