/** \file
  matrix_histogram function.

  $Header: /u/nn/cvsroot/lissom/src/matrixhistogram.h,v 1.2 2001/03/31 22:36:12 jbednar Exp $
*/

#ifndef __MATRIXHISTOGRAM_H__
#define __MATRIXHISTOGRAM_H__

#include "../src/histogram.h"
#include "../src/matrix.h"
#include "../src/genericalgs.h"

namespace  Histo {


/** Generates a histogram by treating each corresponding pair of
    elements from the BinMatrix and the ValueMatrix as a (bin,value)
    pair.  To get the bin number in which to place each value, the
    levels in the BinMatrix are scaled using the given function object
    and then wrapped or cropped (depending on the 'wrap' argument)
    into the range 0..num_bins-1.  The matrix type must be supported
    by mat::elem.  The typical scale object is Scale::LinearScale.  */
template <class BinMatrix, class ValueMatrix, class ScaleType>
Histo::OneDHistogram<> matrix_histogram(const BinMatrix& mb, const ValueMatrix& mv,
					const typename BinMatrix::size_type num_bins,
					const ScaleType& binscale,
					bool wrap=false)
{
  Histo::OneDHistogram<> h(num_bins);
  
  /* Uses matrix interface rather than a 1D-iterator to ensure that
     corresponding elements match. */
  typedef typename BinMatrix::size_type  size_type;
  typedef typename BinMatrix::value_type value_type;
  for   (size_type r=0; r<mb.nrows(); r++)
    for (size_type c=0; c<mb.ncols(); c++) {
      const value_type binval = binscale(mat::elem(mb,r,c));
      const size_type  binnum = size_type(binval);
      //const size_type  binnum = size_type(floor(binval+0.5)); /* Rounds */ //////
      const size_type  bin    = (wrap? Generic::wrap(size_type(0),size_type(num_bins),  size_type(binnum))
				 :     Generic::crop(size_type(0),size_type(num_bins-1),size_type(binnum)));
      h.add(bin,mat::elem(mv,r,c));
    }
  
  return h;
}



} /* namespace Histo */
#endif /* __MATRIXHISTOGRAM_H__ */
