/** \file
  PathSpecification class.
  
  $Header: /u/nn/cvsroot/lissom/src/pathspecification.h,v 1.3 2001/09/02 21:31:13 jbednar Exp $
*/

#ifndef ___PATHSPECIFICATION_H___
#define ___PATHSPECIFICATION_H___

#include "stringutils.h"
#include "stringparser.h"



/** Simple class holding a fully-specified path to an item.
    
  The initial string specification should be of the form
  [name][::name]*[::], i.e. a list of names separated by double
  colons, with no whitespace.

  Operations return empty strings or empty lists if the
  appropriate item is unavailable or undefined.
*/
class PathSpecification {
public:
  typedef StringParser::arglist NameList;

  /** Constructor */
  PathSpecification(const string& s)
    : rooted(s.length()>=2 && s[0]==':' && s[1]==':')
    {
      assert (String::replace_all(s,string(" "),string("")) == s);
      const string with_spaces = String::replace_all(s,string("::"),string(" "));
      
      /* Break into a vector */
      StringParser sp;
      sp.parse(with_spaces,names);
    }

  /** Returns the tail of the path, if any */
  const string last() const
    {  return (names.empty()? string() : names.back());  }

  /** Returns the head of the path, if any */
  const string first() const
    {  return (names.empty()? string() : names.front());  }

  /** Returns true if the path started with the segment separator, i.e. :: */
  const bool  isrooted() const {  return rooted;  }

  /** Returns the entire path */
  const NameList& all() const
    {  return names; }

  /** Returns the path up to (but not including) the tail */
  const NameList allbutlast() const {
    return (names.begin()==names.end()? NameList() : 
	    NameList(names.begin(),names.end()-1));
  }

private:
  NameList names;
  bool rooted;
};


  
#endif /* ___PATHSPECIFICATION_H___ */
