/** \file
    
  Scale namespace: Classes that scale their argument according to some
  internal function.  Currently only linear scaling is implemented.

  $Header: /u/nn/cvsroot/lissom/src/scale.h,v 1.3 2001/03/06 00:50:14 jbednar Exp $
*/


#ifndef __SCALE_H__
#define __SCALE_H__

#include <functional>


/** Namespace for scaling functions or objects, e.g. classes or
    functions that scale (and/or offset, etc.) their one-dimensional
    argument.
*/
namespace Scale {



/** Function object that returns its object as-is.  Can be useful with
    standard library routines; may already be available somewhere in
    the standard library.  */
template<class T=double>
struct Identity : public std::unary_function<T,T> {
  inline T operator() (const T& val) const {  return val;  }
};
 


/** Function object that linearly scales its arguments.  The
  multiplier and offset are public because they will often
  need to be extracted to give to functions which expect
  separate multipliers and offsets rather than a function
  object.
*/
template<class T=double>
struct Linear : public std::unary_function<T,T> {
  T multiplier, offset;

  Linear(const T& mult=T(1), const T& off=T(0))
    : multiplier(mult), offset(off)  {  }
  
  inline T operator() (T val) const 
    {  return val*multiplier + offset;  }
};
 
} /* namespace Scale */
#endif /* __SCALE_H__ */
