/** \file
  SequenceTransform namespace.
  $Header: /u/nn/cvsroot/lissom/src/sequencetransform.h,v 1.3 2001/11/25 08:46:58 jbednar Exp $
*/ 

#ifndef __SEQUENCE_TRANSFORM_H__
#define __SEQUENCE_TRANSFORM_H__

#include "genericalgs.h"


/** Namespace for class hierarchies of function objects transforming
    sequences into new ones. */
namespace SequenceTransform {



/** Base class for a function that applies itself to a container to
    create another container with the same number of elements.
    (The second container may be the same one as the first). */
template <class InContainer, class OutContainer>
class OneD {
public:
  virtual void operator() (const InContainer& in, OutContainer& out) const=0;
};


/** Example of basic thresholding. */
template <class InContainer, class OutContainer>
class Threshold : public OneD<InContainer,OutContainer> {
public:
  typedef typename InContainer::value_type value_type;
  Threshold(const value_type* threshold_=0) : threshold(threshold_) { }
  virtual void operator() (const InContainer& in, OutContainer& out) const
    {  if (threshold) Generic::lower_threshold(in.begin(),in.end(),out.begin(),*threshold,0.0);  }
private:
  const value_type* threshold;
};



} /* namespace SequenceTransform */

#endif /* __SEQUENCE_TRANSFORM_H__ */
