/** \file
  Tristate type.
  
  $Header: /u/nn/cvsroot/lissom/src/tristate.h,v 1.7 2004/08/24 13:56:05 jbednar Exp $
*/

#ifndef ___TRISTATE_H___
#define ___TRISTATE_H___

#include <fstream>
#include <cassert>
#include <string>
using std::string;
#include <sstream>


/**
  Three-valued Boolean type, including Uninitialized as well as the
  usual values.  Uninitialized is useful for denoting a default value
  to be computed automatically.  
*/	
enum Tristate {Uninitialized=-1,False=false,True=true};


/* Double-check assumption */
// check is not allowed under ISO C++
//#if (false!=0)
//#error 'false' must denote zero
//#endif



/** Stream output for a Tristate */
inline std::ostream& operator<<(std::ostream &s, const Tristate& t) {
  switch(t){
  case Uninitialized: return s << "Uninitialized";  
  case False:         return s << "False";          
  case True:          return s << "True";          
  default:            return s << int(t); // Unknown value
  }
}


/** Stream input for a Tristate */
inline std::istream& operator>>(std::istream &s, Tristate& t) {
  string r;
  s >> r;
  if      (r == "Uninitialized") t=Uninitialized;
  else if (r == "False")         t=False;
  else if (r == "True")          t=True;
  else {
    std::istringstream ss(r);
    int x=0;
    ss >> x;
    t=Tristate(x);
  }
  
  return s;
}

#endif /* ___TRISTATE_H___ */
