/** \file
  Typeless class.
  
  $Header: /u/nn/cvsroot/lissom/src/typeless.h,v 1.3 2001/04/01 07:06:01 jbednar Exp $
*/

#ifndef ___TYPELESS_H___
#define ___TYPELESS_H___

#include <string>
using std::string;
#include <iostream>

#include "../src/stringutils.h"
#include "../src/ind_types.h"



/** Abstract base class for a pointer to an object of nominally
    unspecified type.  The operations provided here should either be
    supported by all types, or they should have a guard routine such
    as isnumeric() below.  Binary routines (e.g. operator<) are only
    guaranteed to be supported between two identical underlying types.
    Other combinations may also be supported; if they are not a
    runtime error should be generated (an assertion or error exit).
*/
class Typeless {
public:
  typedef  Typeless self;
  virtual ~Typeless() { }
  
  /** Returns a string representation of the current value of this object */
  virtual string stringrep(String::StreamFormat format=String::StreamFormat()) const=0;
  
  /** Returns a string representation of the current value of this object,
      with extra formatting suitable for user-visible values, if appropriate
      (e.g. quotes around strings).  Unless overridden by a particular class,
      this is the same as stringrep().
  */
  virtual string prettyprint(String::StreamFormat format=String::StreamFormat()) const
    {  return stringrep(format);  }
  
  /** Returns a string representation of the datatype of this object.
      The value is not guaranteed to be anything particularly useful,
      one cannot count on any implementation more specific than the
      one here. */ 
  virtual string typestring() const {  return "Unspecified";  }

  virtual bool operator<  (const Typeless& o) const=0;
  virtual bool operator>  (const Typeless& o) const=0;
  virtual bool operator== (const Typeless& o) const
    {  return !(*this < o || *this > o);  }
  virtual bool operator!= (const Typeless& o) const
    {  return !(*this == o);  }

  virtual Typeless& set(const Typeless& o) =0;
  
  /** Unified return type for numeric values */
  typedef
#ifndef LONG_DOUBLE_UNAVAILABLE
  long
#endif
  double exprtype;

  /** Returns true if this is a type for which numericvalue() is defined */
  virtual bool isnumeric() const {  return false;  }

  /** If the type is numeric, return it as exprtype; else return 0 */
  virtual exprtype numericvalue() const {  return 0;  }

  /** Returns the result of parsing the given string with the given parser */
  virtual Typeless* parse(const StringParser& parser, const string& s) const =0;
  
  /** Constructs an identical copy of self and returns it. */
  virtual Typeless* clone() const=0;

  /** Same as clone except that the result is guaranteed to
      have an independent copy of the underlying value */
  virtual Typeless* duplicate() const=0;
};



#endif /* ___TYPELESS_H___ */
