/* -*-mode: C++; compile-command: "gmake -C../tests boundednumber.test"; -*- */
/**
  \file   Tests the corresponding .h file.
  $Header: /u/nn/cvsroot/lissom/tests/boundednumber.c,v 1.8 2001/04/07 02:19:11 jbednar Exp $
*/

#include "testutils.h"
#include "../src/boundednumber.h"

int main(int, char**)
{
  typedef Bounded::Integer<unsigned char> BoundedI;
  typedef Bounded::Float<float>           BoundedF;

  section("Sizes");
#ifdef _CRAYMPP
  std::cout << "Cray bloats small classes for some reason" << std::endl;
  EXACT_CHECK( int(sizeof(BoundedI)),   8);
  EXACT_CHECK( int(sizeof(BoundedF)),   8);
#else
  EXACT_CHECK( int(sizeof(BoundedI)),   1);
  EXACT_CHECK( int(sizeof(BoundedF)),   4);
#endif
  EXACT_MATCH( int(BoundedI::Max),    255);
  FLOAT_MATCH( 	   BoundedF::Max ,    1.0);

  section("Construction");
  BoundedI x(222);   EXACT_MATCH(int(x.raw()), 222);
  BoundedF y(0.53);  LOOSE_MATCH(    y.raw(),  0.53,   0.0001);
  
  BoundedF z(x);     LOOSE_MATCH(    z.raw(),  0.8706, 0.0001);
  BoundedI w(z);     EXACT_MATCH(int(w.raw()), 222);
                     LOOSE_MATCH(    w.mag(),  0.8706, 0.0001);
  BoundedF q(w);     LOOSE_MATCH(    q.raw(),  0.8706, 0.0001);

  section("Assignment");
  x=-2;              EXACT_MATCH(int(x.raw()), 0);
  w=302;             EXACT_MATCH(int(w.raw()), 255);
  q=0.999902;        LOOSE_MATCH(    q.raw(),  0.999902, 0.0001);

  section("Arithmetic");
  BoundedI a(128);   EXACT_MATCH(int(a.raw()), 128);
  BoundedI b(30);    EXACT_MATCH(int(b.raw()), 30);
  a*=0.5;            EXACT_MATCH(int(a.raw()), 64);
  BoundedI c(a+b);   EXACT_MATCH(int(c.raw()), 94);
  a*=3;              EXACT_MATCH(int(a.raw()), 192);
  b*=3;              EXACT_MATCH(int(b.raw()), 90);

  section("Cropping arithmetic");
  c*=12345;          EXACT_MATCH(int(c.raw()), 255);
                     EXACT_MATCH(int((a+b).raw()), 255);

  return count_errors();
}
