/* -*-mode: C++; compile-command: "gmake -C../tests boundingbox.test"; -*- */
/**
  \file   Tests the corresponding .h file.
  $Header: /u/nn/cvsroot/lissom/tests/boundingbox.c,v 1.11 2001/11/24 22:17:04 jbednar Exp $
*/

#include "testutils.h"
#include "pixelutils.h"
#include "../src/matriximage.h"
#include "../src/pixel.h"
#include "../src/pnm_io.h"
#include "../src/colorlookup.h"
#include "../src/matrixadapter.h"
#include "../src/boundingbox.h"

#ifdef EXPLICIT_INSTANTIATION_FOR_STATICS
template class Plot::HSVPixel<Plot::Pixel::FComponent,Plot::Pixel::HueComponent>;
#endif 


int main(int, char**)
{
  section("Axis-aligned rectangle of double");
  Boundary::AARBoundingBox<double> rd(2,3,10,20);
  EXACT_MATCH( rd.inside(4,9),      true  );  
  EXACT_MATCH( rd.inside(2,3),      true  );  
  EXACT_MATCH( rd.inside(2,2),      false );  
  EXACT_MATCH( rd.inside(10,20),    true  );  
  EXACT_MATCH( rd.inside(10.01,20), false );  
  EXACT_MATCH( rd.inside(11,20),    false );  
  EXACT_MATCH( rd.inside(10,30),    false );  
    
  section("Axis-aligned rectangle of int");
  Boundary::AARBoundingBox<double> ri(2,3,10,20);
  EXACT_MATCH( ri.inside(4,9),      true  );
  EXACT_MATCH( ri.inside(2,3),      true  );
  EXACT_MATCH( ri.inside(2,2),      false );
  EXACT_MATCH( ri.inside(10,20),    true  );
  EXACT_MATCH( ri.inside(11,20),    false );
  EXACT_MATCH( ri.inside(10,30),    false );
  
  section("Axis-aligned ellipse of double");
  Boundary::BoundingEllipse<double> ed(5.5,10.5,5);
  EXACT_MATCH( ed.inside(5,10),     true  );
  EXACT_MATCH( ed.inside(8,10),     true  );
  EXACT_MATCH( ed.inside(8.5,10),   true  );
  EXACT_MATCH( ed.inside(8,11),     true  );
  EXACT_MATCH( ed.inside(0,0),      false );
  
  section("Axis-aligned ellipse of int");
  Boundary::BoundingEllipse<int> ei(5,10,5);
  EXACT_MATCH( ei.inside(5,10),     true  );
  EXACT_MATCH( ei.inside(8,10),     true  );
  EXACT_MATCH( ei.inside(int(8.1),10),   true  );
  EXACT_MATCH( ei.inside(8,11),     true  );
  EXACT_MATCH( ei.inside(0,0),      false );
  
  section("Circle of double");
  Boundary::BoundingCircle<double> cd(5.5,10.5,5);
  EXACT_MATCH( cd.inside(5,10),     true  );
  EXACT_MATCH( cd.inside(8,10),     true  );
  EXACT_MATCH( cd.inside(8.5,10),   true  );
  EXACT_MATCH( cd.inside(8,11),     true  );
  EXACT_MATCH( cd.inside(0,0),      false );
  
  section("Circle of int");
  Boundary::BoundingCircle<int> ci(5,10,4.5);
  EXACT_MATCH( ci.inside(5,10),     true  );
  EXACT_MATCH( ci.inside(8,10),     true  );
  EXACT_MATCH( ci.inside(int(8.1),10),   true  );
  EXACT_MATCH( ci.inside(8,11),     true  );
  EXACT_MATCH( ci.inside(0,0),      false );
  
  section("Intersection between a circle and a rectangle");
  Boundary::AARBoundingBox<double> rd2(6,3,10,10);
  const Boundary::Intersection<double,bool,double,
    Boundary::BoundingCircle<double>,
    Boundary::AARBoundingBox<double> >  rd2icd(cd,rd2);
  
  EXACT_MATCH( rd2icd.inside(5,5),      false );
  EXACT_MATCH( rd2icd.inside(5,10),     false );
  EXACT_MATCH( rd2icd.inside(8,10),     true  );
  EXACT_MATCH( rd2icd.inside(8.1,10),   true  );
  EXACT_MATCH( rd2icd.inside(8,11),     false );
  EXACT_MATCH( rd2icd.inside(0,0),      false );
  
  section("Image construction");
  typedef MatrixType<float>::rectangular Matrix;
  Matrix g(8,20);
  for (Matrix::size_type r=0; r<g.nrows(); r++)
    for (Matrix::size_type c=0; c<g.ncols(); c++)
      /* Can test default colors by subtracting each of these from 1.0 or 0.25 */
      g[r][c]=ci.inside(r,c)*sin(r*100000+c);
  
  section("Image file output");
  const int border=3;
  const double size_scale=6;
  Plot::MatrixImage<> image(g,size_scale,border,Plot::RYWColorLookup<>() );
  image.outline_boundary(ci, 2);
  image.outline_boundary(ci, 1, Plot::HSVPixel<>(0.4,0.8,0.9) );
  PPM_WRITE("bounded",image,"Bounded by a circle");
  
  return count_errors();
}







