/* -*-mode: C++; compile-command: "cd ../tests ; rm -f expressionparser ; g++ -g -Wall expressionparser.c -o expressionparser; ./expressionparser; diff reference/parameter.txt output/parameter.txt"; -*- */
/**
  \file   Tests expressionparser.h and parametermap.h
  $Header: /u/nn/cvsroot/lissom/tests/expressionparser.c,v 1.33 2001/04/08 04:16:44 jbednar Exp $
*/

#include "testutils.h"
#include "msgutils.h"

#include "../src/parameter.h"
#include "../src/parametermap.h"
#include "../src/expressionparser.h"
#include "../src/polymorph.h"

#ifdef EXPLICIT_INSTANTIATION_FOR_STATICS
template class Msg::LevelHandler<Msg::MessageLevel,Msg::NumMessageLevels>;
template class PolymorphBase<Tristate>;
template class PolymorphBase<int>;
template class PolymorphBase<double>;
template class PolymorphBase<string>;
template class ParameterDefinition<int>;
#endif 



/* Constants */
const double init_f=0.5;



/* Sample parameters */
Tristate bparam=True;
double fParam=init_f;
double fParam2=0;
int    i_param=6;
int    i_param2=9;
string usparam("initialstring");



/* Sample setfn */
int setfn_called=0;
SETFN_DECLARE2(i_param_setfn2);
cmdstat i_param_setfn2(const string&, Typeless& existing, const Typeless& requested)
{
  existing.set(requested);
  setfn_called++;
  return 0;
}


/* Useful macros */
#define PARSE(typechar,expr)  Parse(blackboard,#expr).cmd ## typechar()
#define PARSE_EXACT_MATCH(type,typechar,expr)  EXACT_MATCH( type(PARSE(typechar,expr)), type(expr) )
#define PARSE_FLOAT_MATCH(expr)           FLOAT_MATCH( PARSE(f,expr), expr )
#define SET(name,expr) blackboard.local_copy(blackboard.lookup(#name)).set(ParamStringParser<>(blackboard),#expr)
#define SET_EXACT_MATCH(name,expr)        SET(name,expr) ; EXACT_MATCH( name, expr )
#define SET_FLOAT_MATCH(name,expr)        SET(name,expr) ; FLOAT_MATCH( name, expr )

#define DECLARE_PARAM(type,name_str,location) \
ParamType(make_Polymorph(location),name_str)
//ParamType(name_str,type,false,(void*)location)
#define PARAM(type,name) \
DECLARE_PARAM(type,#name,&name)
#define PARAM_I(type,name,lower,upper,doc) \
blackboard.define_param(PARAM(type,name).add_doc(doc).add_lower_bound(lower).add_upper_bound(upper))
#define PARAM_N(type,name,doc) \
blackboard.define_param(PARAM(type,name).add_doc(doc))


/** FLOAT_MATCH for a pointer argument, expecting a non-null pointer */
/* The implementation is a bit of a hack to ensure that these messages
   will be treated the same as other MATCH messages. */
#define FLOAT_PTR_MATCH(valueptr,memfuncall,expected)\
  if (valueptr) {  FLOAT_MATCH( (*valueptr)memfuncall, expected );  }\
  else          {  EXACT_MATCH( #valueptr, "(any nonzero value)" ); }


// Actually not CRAY-specific, just 64-bit-specific, but don't have a handy test for
// the latter at present  
#ifdef _CRAY  
#define INT_MIN_STR "-35184372088832"
#define INT_MAX_STR "35184372088831"
#else
#define INT_MIN_STR "-2147483648"
#define INT_MAX_STR "2147483647"
#endif
     
     
#undef SETFNOBJ_DEFINE2
#define SETFNOBJ_DEFINE2(cargs,paramname,fnname) \
blackboard.lookup(#paramname).add_setfn(new setfnobj2_ ## fnname(cargs))

#undef SETFN_DEFINE2
#define SETFN_DEFINE2(paramname,fnname) SETFNOBJ_DEFINE2(,paramname,fnname)


int main(int, char**)
{
  /* For pretty */
  Polymorph<Tristate>::set_typestring("Boolean");
  Polymorph<int>::set_typestring("Integer");
  Polymorph<double>::set_typestring("Float");
  Polymorph<string>::set_typestring("String");


  section("Construction");
  typedef ParameterDefinition<>        ParamType;
  typedef ParameterMap<ParamType>      BlackboardType;
  typedef ExpressionParser<>           Parse;
  Msg::LevelHandler<>::set_default_instance(&Msg::counting_message_handler);
  BlackboardType blackboard;
  EXACT_MATCH( blackboard.check_all(),  0);
  Msg::check_for_unexpected_messages();


  section("Constant definition");
  blackboard.add_constants();
  EXACT_MATCH( blackboard.check_all(),  0);
  Msg::check_for_unexpected_messages();


  section("Parameter definition");
  PARAM_I(PARAM_3BOOL, bparam, Uninitialized,True, "The documentation for %s,\nwhich takes two lines.");
  PARAM_I(PARAM_DOUBLE,fParam, 0,            1,    "The documentation for %s.\n");
  PARAM_I(PARAM_DOUBLE,fParam2,-100,         100,  "Some documentation for %s.\n");
  PARAM_N(PARAM_USTRING,usparam,                   "The\nthree\nline documentation for %s.");
  PARAM_I(PARAM_INT,   i_param,0,            12,   "Documentation for %s.");
  Msg::check_for_unexpected_messages();

#define FDEFAULT 0.01*i_param
  blackboard.lookup("fParam").add_default_expr("0.01*i_param");
  SETFN_DEFINE2(i_param,i_param_setfn2);
  EXACT_MATCH( blackboard.check_all(),  0);
  Msg::check_for_unexpected_messages();


  section("Parameter lookup and expression parsing");
  PARSE_EXACT_MATCH( double, f,             fParam/2-0.1);
  PARSE_EXACT_MATCH( int,    i,             i_param*2+1);
  EXACT_MATCH( string(PARSE(s,...${usparam}...)),string("...initialstring..."));
  const Typeless* il = blackboard.getptr("i_param");
  FLOAT_PTR_MATCH(il,.numericvalue(),6.0);
  Msg::check_for_unexpected_messages();


  section("String formatting directives");
  EXACT_MATCH( string(PARSE(s,${i_param})),   string("6"));
  EXACT_MATCH( string(PARSE(s,${05i_param})), string("00006"));
  EXACT_MATCH( string(PARSE(s,${fParam})),    string("0.5"));
  EXACT_MATCH( string(PARSE(s,${06fParam})),  string("0000.5"));
  EXACT_MATCH( string(PARSE(s,${6fParam})),   string("   0.5"));
  Msg::check_for_unexpected_messages();


  section("Updating default values");
  ParamStringParser<> sp(blackboard);
  blackboard.update_all_default_values(sp);
  EXACT_MATCH( fParam,                        FDEFAULT );
  EXACT_MATCH( Parse(blackboard,"fParam").cmdf(), fParam );
  Msg::check_for_unexpected_messages();


  section("Explicit setting");
#define FSETTING 0.25
  SET_FLOAT_MATCH(fParam,0.25);
  SET(usparam,A string value);
  EXACT_MATCH( usparam,             string("A string value"));
  SET_EXACT_MATCH(i_param,5);

  EXACT_MATCH( setfn_called,        1     );
  ParameterDefinition<>::changes_verbose=true;
  SET_EXACT_MATCH(i_param,10);
  EXACT_MATCH( setfn_called,        2     );
  EXACT_MATCH( Msg::last_notify,    string("Changed parameter i_param to 10") );
  EXACT_MATCH( i_param,             10    );
  SET_FLOAT_MATCH(fParam2,0.25/i_param);
  SET_FLOAT_MATCH(fParam2,-0.25/i_param);
  SET_FLOAT_MATCH(fParam2,-1);
  SET_FLOAT_MATCH(fParam2,0.25);
  SET_EXACT_MATCH(bparam,true);
  SET_EXACT_MATCH(bparam,false);
  SET_EXACT_MATCH(bparam,Uninitialized);
  EXACT_MATCH( Msg::last_notify,    string("Changed parameter bparam to Uninitialized") );
  ParameterDefinition<>& param_bparam = blackboard.lookup("bparam");
  EXACT_MATCH( param_bparam.datavalue().stringrep(), "Uninitialized");
  EXACT_MATCH( param_bparam.stringrep(),             "Uninitialized");
  SET_EXACT_MATCH(bparam,-1);
  SET_EXACT_MATCH(bparam,Uninitialized);
  blackboard.init_hook();
  Msg::check_for_unexpected_messages();
  

  section("Updating default values after setting");
  blackboard.update_all_default_values(sp);
  EXACT_MATCH( fParam,              FSETTING);
  PARSE_EXACT_MATCH(double, f,      fParam);
  PARSE_EXACT_MATCH(int,    i,      i_param);
  EXACT_MATCH( blackboard.check_all(),  0     );
  Msg::check_for_unexpected_messages();


  section("Error handling");
  EXACT_MATCH( Msg::warnings,       0);
  blackboard.lookup("bogus").add_doc("Dummy documentation");
  Msg::expected_error("No parameter (or constant) `bogus' is defined");
  SET(bogus,5);
  Msg::expected_error("No parameter (or constant) `bogus' is defined");
  Msg::check_for_unexpected_messages();
  EXACT_MATCH( int(PARSE(i,bogus)), -1);
  Msg::expected_error("No parameter (or constant) `bogus' is defined");
  EXACT_MATCH( int(PARSE(i,_b)),    -1);
  Msg::expected_error("Can't parse expression: `_b'");
  SET(i_param,2e15); // Value intentionally too large for most integer types
  Msg::expected_warning("Value 2e+15 outside the representable range of an integer (" INT_MIN_STR "," INT_MAX_STR "); may cause overflow or underflow");
// As above, the real test is for a 64-bit machine, not CRAY machines in particular
#ifdef _CRAY  
  Msg::expected_error("i_param too large (2000000000000000 > 12)");
#else
  Msg::expected_error("i_param too small (" INT_MIN_STR " < 0)",1,true);
#endif
  blackboard.define_param(DECLARE_PARAM(PARAM_INT,"para#2",&i_param2));
  Msg::expected_warning("Invalid characters in parameter name `para#2'; can't define it");
  Msg::check_for_unexpected_messages();
  

  section("Free-form parsing");
  Parse st(blackboard,"&True,");
  EXACT_MATCH( st.empty(),                false );
  EXACT_MATCH( st.consumeifpresent(","),  false );
  EXACT_MATCH( st.empty(),                false );
  EXACT_MATCH( st.consumeifpresent("&"),  true  );
  EXACT_MATCH( bool(st.getnumericval()),  true  );
  EXACT_MATCH( st.consumeifpresent(","),  true  );
  EXACT_MATCH( st.empty(),                true  );
  Msg::check_for_unexpected_messages();

  
  section("File output to ./output/parameter.txt");
  FILE* file=fopen("output/parameter.txt","w+");
  blackboard.print_parameters_doc(file);
  blackboard.print_all(file);
  fprintf(file,"\n\nConstants:\n");
  blackboard.print_constants(file);
  blackboard.print_constants_doc(file);
  blackboard.print_parameters(file);
  fprintf(file,"\n\nMessages:\n%s",Msg::messages.c_str());
  Msg::check_for_unexpected_messages();


  section("Nested blackboards");
  SET(i_param,3);
  BlackboardType blackboard2(&blackboard,"child");
  BlackboardType blackboard2_a(&blackboard2,"sub_child_a");
  BlackboardType blackboard2_b(&blackboard2,"sub_child_b");
  FLOAT_MATCH(  blackboard.lookup("i_param").datavalue().numericvalue(), 3.0 );
  FLOAT_MATCH( blackboard2.lookup("i_param").datavalue().numericvalue(), 3.0 );
  /* Note that the ParamStringParser is used to parse the expression
     itself; any expression-related lookup is done in the blackboard
     specified therein rather than the one on which the 'set' is
     performed. */
  blackboard2.local_copy(blackboard2.lookup("i_param")).set(ParamStringParser<>(blackboard),"9");
  FLOAT_MATCH(  blackboard.lookup("i_param").datavalue().numericvalue(), 3.0 );
  FLOAT_MATCH( blackboard2.lookup("i_param").datavalue().numericvalue(), 9.0 );
  Msg::check_for_unexpected_messages();
  
  BlackboardType& bp   = blackboard.lookup_map("");
  BlackboardType& bpc  = blackboard.lookup_map("child");
  BlackboardType& bpca = blackboard.lookup_map("child::sub_child_a");
  BlackboardType& bpcb = blackboard.lookup_map("child::sub_child_b");
  EXACT_MATCH( bpcb.path_name(), string("child::sub_child_b"));  
  bpcb.local_copy(bpcb.lookup("i_param")).set(ParamStringParser<>(blackboard2),"i_param+3");

  FLOAT_MATCH(  bp.lookup("i_param").datavalue().numericvalue(),3.0);
  FLOAT_MATCH( bpc.lookup("i_param").datavalue().numericvalue(),9.0);
  FLOAT_MATCH(bpca.lookup("i_param").datavalue().numericvalue(),9.0);
  FLOAT_MATCH(bpcb.lookup("i_param").datavalue().numericvalue(),12.0);

  bpc.local_copy(bpc.lookup("i_param")).set(ParamStringParser<>(blackboard),"i_param+1");
  FLOAT_MATCH(  bp.lookup("i_param").datavalue().numericvalue(),3.0);
  FLOAT_MATCH( bpc.lookup("i_param").datavalue().numericvalue(),4.0);
  FLOAT_MATCH(bpca.lookup("i_param").datavalue().numericvalue(),4.0);
  FLOAT_MATCH(bpcb.lookup("i_param").datavalue().numericvalue(),12.0);
  Msg::check_for_unexpected_messages();

  blackboard.lookup_map("unknown");
  Msg::expected_error("Could not locate parameter set 'unknown'; using default set");


  section("Setting multiple parameters");
  EXACT_MATCH(blackboard.set_matching(ParamStringParser<>(blackboard),"child::b*::i_param","11"), 0);
  FLOAT_MATCH(    bp.lookup("i_param").datavalue().numericvalue(),3.0);
  FLOAT_MATCH(   bpc.lookup("i_param").datavalue().numericvalue(),4.0);
  FLOAT_MATCH(  bpca.lookup("i_param").datavalue().numericvalue(),4.0);
  FLOAT_MATCH(  bpcb.lookup("i_param").datavalue().numericvalue(),12.0);
  EXACT_MATCH(blackboard.set_matching(ParamStringParser<>(blackboard),"child::*gg*::i_param","8"), 0);
  FLOAT_MATCH(    bp.lookup("i_param").datavalue().numericvalue(),3.0);
  FLOAT_MATCH(   bpc.lookup("i_param").datavalue().numericvalue(),4.0);
  FLOAT_MATCH(  bpca.lookup("i_param").datavalue().numericvalue(),4.0);
  FLOAT_MATCH(  bpcb.lookup("i_param").datavalue().numericvalue(),12.0);
  EXACT_MATCH(blackboard.set_matching(ParamStringParser<>(blackboard),"child::*g::i_param","11"), 0);
  FLOAT_MATCH(    bp.lookup("i_param").datavalue().numericvalue(),3.0);
  FLOAT_MATCH(   bpc.lookup("i_param").datavalue().numericvalue(),4.0);
  FLOAT_MATCH(  bpca.lookup("i_param").datavalue().numericvalue(),4.0);
  FLOAT_MATCH(  bpcb.lookup("i_param").datavalue().numericvalue(),12.0);
  EXACT_MATCH(blackboard.set_matching(ParamStringParser<>(blackboard),"child::*::i_param","8"), 2);
  FLOAT_MATCH(    bp.lookup("i_param").datavalue().numericvalue(),3.0);
  FLOAT_MATCH(   bpc.lookup("i_param").datavalue().numericvalue(),4.0);
  FLOAT_MATCH(  bpca.lookup("i_param").datavalue().numericvalue(),8.0);
  FLOAT_MATCH(  bpcb.lookup("i_param").datavalue().numericvalue(),8.0);
  EXACT_MATCH(blackboard.set_matching(ParamStringParser<>(blackboard),"child::sub*::i_param","11"), 2);
  FLOAT_MATCH(    bp.lookup("i_param").datavalue().numericvalue(),3.0);
  FLOAT_MATCH(   bpc.lookup("i_param").datavalue().numericvalue(),4.0);
  FLOAT_MATCH(  bpca.lookup("i_param").datavalue().numericvalue(),11.0);
  FLOAT_MATCH(  bpcb.lookup("i_param").datavalue().numericvalue(),11.0);
  EXACT_MATCH(blackboard.set_matching(ParamStringParser<>(blackboard),"child::*child*::i_param","8"), 2);
  FLOAT_MATCH(    bp.lookup("i_param").datavalue().numericvalue(),3.0);
  FLOAT_MATCH(   bpc.lookup("i_param").datavalue().numericvalue(),4.0);
  FLOAT_MATCH(  bpca.lookup("i_param").datavalue().numericvalue(),8.0);
  FLOAT_MATCH(  bpcb.lookup("i_param").datavalue().numericvalue(),8.0);
  EXACT_MATCH(blackboard.set_matching(ParamStringParser<>(blackboard),"*ild::*::i_param","11"), 2);
  FLOAT_MATCH(    bp.lookup("i_param").datavalue().numericvalue(),3.0);
  FLOAT_MATCH(   bpc.lookup("i_param").datavalue().numericvalue(),4.0);
  FLOAT_MATCH(  bpca.lookup("i_param").datavalue().numericvalue(),11.0);
  FLOAT_MATCH(  bpcb.lookup("i_param").datavalue().numericvalue(),11.0);
  EXACT_MATCH(blackboard.set_matching(ParamStringParser<>(blackboard),"*::*::i_param","2"), 2);
  FLOAT_MATCH(    bp.lookup("i_param").datavalue().numericvalue(),3.0);
  FLOAT_MATCH(   bpc.lookup("i_param").datavalue().numericvalue(),4.0);
  FLOAT_MATCH(  bpca.lookup("i_param").datavalue().numericvalue(),2.0);
  FLOAT_MATCH(  bpcb.lookup("i_param").datavalue().numericvalue(),2.0);
  EXACT_MATCH(blackboard.set_matching(ParamStringParser<>(blackboard),"*::i_param","9"), 1);
  FLOAT_MATCH(    bp.lookup("i_param").datavalue().numericvalue(),3.0);
  FLOAT_MATCH(   bpc.lookup("i_param").datavalue().numericvalue(),9.0);
  FLOAT_MATCH(  bpca.lookup("i_param").datavalue().numericvalue(),2.0);
  FLOAT_MATCH(  bpcb.lookup("i_param").datavalue().numericvalue(),2.0);
  EXACT_MATCH(blackboard.set_matching(ParamStringParser<>(blackboard),"*::sub_child_a::i_param","1"), 1);
  FLOAT_MATCH(    bp.lookup("i_param").datavalue().numericvalue(),3.0);
  FLOAT_MATCH(   bpc.lookup("i_param").datavalue().numericvalue(),9.0);
  FLOAT_MATCH(  bpca.lookup("i_param").datavalue().numericvalue(),1.0);
  FLOAT_MATCH(  bpcb.lookup("i_param").datavalue().numericvalue(),2.0);
  EXACT_MATCH(blackboard.set_matching(ParamStringParser<>(blackboard),"i_param","0"), 1);
  FLOAT_MATCH(    bp.lookup("i_param").datavalue().numericvalue(),0.0);
  FLOAT_MATCH(   bpc.lookup("i_param").datavalue().numericvalue(),9.0);
  FLOAT_MATCH(  bpca.lookup("i_param").datavalue().numericvalue(),1.0);
  FLOAT_MATCH(  bpcb.lookup("i_param").datavalue().numericvalue(),2.0);
  EXACT_MATCH(blackboard.set_matching(ParamStringParser<>(blackboard),"nonexistent::i_param","0"), 0);
  FLOAT_MATCH(    bp.lookup("i_param").datavalue().numericvalue(),0.0);
  FLOAT_MATCH(   bpc.lookup("i_param").datavalue().numericvalue(),9.0);
  FLOAT_MATCH(  bpca.lookup("i_param").datavalue().numericvalue(),1.0);
  FLOAT_MATCH(  bpcb.lookup("i_param").datavalue().numericvalue(),2.0);

  
  section("ParamStringParser arglist");
  {
    ParamStringParser<>::arglist args;
    sp.parse("I am the cheese.",args);
    EXACT_MATCH( int(args.size()), 4);
    if (args.size()>=4) {
      EXACT_MATCH( args[0], string("I"));
      EXACT_MATCH( args[1], string("am"));
      EXACT_MATCH( args[2], string("the"));
      EXACT_MATCH( args[3], string("cheese."));
    }
  }
  {
    ParamStringParser<>::arglist args;
    sp.parse("'I aye' am 'not the' \"cheese .\"",args);
    EXACT_MATCH( int(args.size()), 4);
    if (args.size()>=4) {
      EXACT_MATCH( args[0], string("I aye"));
      EXACT_MATCH( args[1], string("am"));
      EXACT_MATCH( args[2], string("not the"));
      EXACT_MATCH( args[3], string("cheese ."));
    }
  }

  
  return count_errors();
}
