/* -*-mode: C++; compile-command: "cd ../tests ; rm -f exprparser ; g++ -g -Wall exprparser.c -o exprparser; ./exprparser"; -*- */
/**
  \file   Tests exprparser.h
  $Header: /u/nn/cvsroot/lissom/tests/exprparser.c,v 1.12 2001/12/11 19:57:10 jbednar Exp $
*/

#include "testutils.h"
#include "msgutils.h"

#include "../src/exprparser.h"

#ifdef EXPLICIT_INSTANTIATION_FOR_STATICS
template class Msg::LevelHandler<Msg::MessageLevel,Msg::NumMessageLevels>;
template class PolymorphBase<Tristate>;
template class PolymorphBase<int>;
template class PolymorphBase<double>;
template class PolymorphBase<string>;
#endif 


/* Useful macros */
#define PARSE(typechar,expr)  Parse(#expr).cmd ## typechar()
#define PARSE_EXACT_MATCH(type,typechar,expr)  EXACT_MATCH( type(PARSE(typechar,expr)), type(expr) )
#define PARSE_FLOAT_MATCH(expr)                FLOAT_MATCH( PARSE(f,expr), expr )
#define PARSE_SIN_MATCH(a,b)                   FLOAT_MATCH( PARSE(f,((a)@;(b))), (a)*sin(b) )
#define PARSE_COS_MATCH(a,b)                   FLOAT_MATCH( PARSE(f,((a)@:(b))), (a)*cos(b) )

#define PI 3.14159265358979323846
  
int main(int, char**)
{
  section("Construction");
  typedef ExprParser<>  Parse;
  Msg::LevelHandler<>::set_default_instance(&Msg::counting_message_handler);
  
  section("Expression precedence and parsing");
  PARSE_FLOAT_MATCH(2.0);
  PARSE_FLOAT_MATCH(2.0+3.0);
  PARSE_FLOAT_MATCH(2.0*3.0);
  PARSE_FLOAT_MATCH(2.0/3.0);
  PARSE_FLOAT_MATCH(2.0+2.0-3.0*9.0);
  PARSE_FLOAT_MATCH(-3.0*7.0-2.0*3.0*4.0*0.5);
  PARSE_FLOAT_MATCH(0.1-0.5-8.0*0.3+2.0);
  PARSE_FLOAT_MATCH(-3.0*7.0-2.0*3.0/4.0*0.5);
  PARSE_FLOAT_MATCH(0.1/0.5/8.0*0.3+2.0);
  PARSE_FLOAT_MATCH(0.1/8.0*0.3+2.0);
  PARSE_FLOAT_MATCH(6.0/3.0*2.0);
  PARSE_FLOAT_MATCH(1+6.0/3.0*2.0);
  PARSE_FLOAT_MATCH(2.0+3.0);
  PARSE_FLOAT_MATCH(2.0*(3.0));
  PARSE_FLOAT_MATCH(2.0/3.0);
  PARSE_FLOAT_MATCH(2.0+(2.0-3.0)*9.0);
  PARSE_FLOAT_MATCH(-3.0*(7.0-(2.0*3.0))*4.0*(0.5));
  PARSE_FLOAT_MATCH(0.1-0.5-(8.0*0.25)+2.0);
  PARSE_FLOAT_MATCH(0.5-(8.0*0.25)+2.0);
  PARSE_FLOAT_MATCH((8.0*0.25)+2.0);
  PARSE_FLOAT_MATCH((-3.0*7.0-2.0)*(3.0/4.0*0.5));
  PARSE_FLOAT_MATCH(0.1/((0.5/8.0)*(0.3+2.0)));
  PARSE_FLOAT_MATCH(0.1/(((8.0)*0.3)+2.0));
  PARSE_FLOAT_MATCH(6.0/((3.0)*2.0));
  PARSE_FLOAT_MATCH(1+(6.0/3.0)*(2.0));
  PARSE_SIN_MATCH(2,0);
  PARSE_SIN_MATCH(2,PI/2);
  PARSE_SIN_MATCH(-0.5,PI);
  PARSE_SIN_MATCH(-0.5,79.1);
  PARSE_COS_MATCH(2,0);
  PARSE_COS_MATCH(0.5,PI/2);
  PARSE_COS_MATCH(-0.5,PI);
  PARSE_COS_MATCH(-0.5,79.1);
  Msg::check_for_unexpected_messages();

  /* Note that (at least when this was written) the parentheses are
     required here, unlike in C, since the precedence order differs. */
  PARSE_FLOAT_MATCH((0<5)+2);
  PARSE_FLOAT_MATCH((5>0)+2);
  EXACT_MATCH( int(PARSE(i,(1<?2)+2)), 3);
  EXACT_MATCH( int(PARSE(i,(5>?0)+2)), 7);

  /* Other datatypes */
  EXACT_MATCH( string(PARSE(s,str)),string("str"));

  
  section("Free-form parsing");
  Parse st("&,3.9--5.8");
  EXACT_MATCH( st.empty(),                false );
  EXACT_MATCH( st.consumeifpresent(","),  false );
  EXACT_MATCH( st.empty(),                false );
  EXACT_MATCH( st.consumeifpresent("&"),  true  );
  EXACT_MATCH( st.consumeifpresent(","),  true  );
  FLOAT_MATCH( double(st.getnumericval()),3.9   );
  EXACT_MATCH( st.consumeifpresent("--"), true  );
  EXACT_MATCH( st.empty(),                false );
  FLOAT_MATCH( double(st.getnumericval()),5.8   );
  EXACT_MATCH( st.empty(),                true  );
  Msg::check_for_unexpected_messages();
  

  section("Error handling");
  Msg::check_for_unexpected_messages();
  EXACT_MATCH( Msg::errors,         0);
  EXACT_MATCH( Msg::warnings,       0);
  EXACT_MATCH( int(PARSE(i,_b)),    -1);  Msg::expected_error("Can't parse expression: `_b'");
  EXACT_MATCH( int(PARSE(i,4^3)),   -1);  Msg::expected_error("Unknown operator ^");
  EXACT_MATCH( int(PARSE(i,4*(^3))),-4);  Msg::expected_error("Can't parse expression: `^3)'");
  Msg::check_for_unexpected_messages();
 

  return count_errors();
}
