/* -*-mode: C++; compile-command: "gmake -C../tests genericalgs.test"; -*- */
/**
  \file   Tests the corresponding .h file.
  $Header: /u/nn/cvsroot/lissom/tests/genericalgs.c,v 1.6 2001/03/10 23:50:56 jbednar Exp $
*/

#include "testutils.h"
#include <vector>
#include <functional>
#include "../src/genericalgs.h"

/* Simple class which counts calls to its destructor */
struct Int {
  Int(int i_) : i(i_) { }
  ~Int() {  destructions++;  }
  inline int value() const { return i;  }
  int i;
  static int destructions;
};
int Int::destructions=0;


int main(int, char**)
{
  section("Floating range 0,1");
  FLOAT_MATCH(Generic::wrap(0.0,1.0,   		    0.1),    0.1);
  FLOAT_MATCH(Generic::wrap(0.0,1.0,   		 -999.2),    0.8);
  FLOAT_MATCH(Generic::wrap(0.0,1.0,   		 1102.3),    0.3);
  FLOAT_MATCH(Generic::crop(0.0,1.0,   		    0.1),    0.1);
  FLOAT_MATCH(Generic::crop(0.0,1.0,   		 -999.2),    0.0);
  FLOAT_MATCH(Generic::crop(0.0,1.0,   		 1102.3),    1.0);
  
  section("Floating range -1,1");
  FLOAT_MATCH(Generic::wrap(-1.0,1.0,         	    0.0),    0.0);
  FLOAT_MATCH(Generic::wrap(-1.0,1.0,         	    1.1),   -0.9);
  FLOAT_MATCH(Generic::wrap(-1.0,1.0,         	   -1.2),    0.8);
  FLOAT_MATCH(Generic::wrap(-1.0,1.0,         	   -0.3),   -0.3);
  FLOAT_MATCH(Generic::crop(-1.0,1.0,         	    0.0),    0.0);
  FLOAT_MATCH(Generic::crop(-1.0,1.0,         	    1.1),    1.0);
  FLOAT_MATCH(Generic::crop(-1.0,1.0,         	   -1.2),   -1.0);
  FLOAT_MATCH(Generic::crop(-1.0,1.0,         	   -0.3),   -0.3);
  
  section("Floating range 0,360");
  FLOAT_MATCH(Generic::wrap(0.0,360.0,        0.0-2*360),    0.0);
  FLOAT_MATCH(Generic::wrap(0.0,360.0,       90.0+2*360),   90.0);
  FLOAT_MATCH(Generic::wrap(0.0,360.0,      450.0+2*360),   90.0);
  FLOAT_MATCH(Generic::wrap(0.0,360.0,      -90.0+2*360),  270.0);
  FLOAT_MATCH(Generic::crop(0.0,360.0,        0.0-2*360),    0.0);
  FLOAT_MATCH(Generic::crop(0.0,360.0,       90.0+2*360),  360.0);
  FLOAT_MATCH(Generic::crop(0.0,360.0,      450.0+2*360),  360.0);
  FLOAT_MATCH(Generic::crop(0.0,360.0,      -90.0+2*360),  360.0);
  
  section("Floating range -180,180");
  FLOAT_MATCH(Generic::wrap(-180.0,180.0,   0.0+789*360),    0.0);
  FLOAT_MATCH(Generic::wrap(-180.0,180.0,  90.0-789*360),   90.0);
  FLOAT_MATCH(Generic::wrap(-180.0,180.0, 450.0-789*360),   90.0);
  FLOAT_MATCH(Generic::wrap(-180.0,180.0, -90.0-789*360),  -90.0);
  FLOAT_MATCH(Generic::crop(-180.0,180.0,   0.0+789*360),  180.0);
  FLOAT_MATCH(Generic::crop(-180.0,180.0,  90.0-789*360), -180.0);
  FLOAT_MATCH(Generic::crop(-180.0,180.0, 450.0-789*360), -180.0);
  FLOAT_MATCH(Generic::crop(-180.0,180.0, -90.0-789*360), -180.0);
  
  section("Integer range -180,180");
  EXACT_MATCH(Generic::wrap(-180,180, 	      0+789*360),      0);
  EXACT_MATCH(Generic::wrap(-180,180, 	     90-789*360),     90);
  EXACT_MATCH(Generic::wrap(-180,180, 	    450-789*360),     90);
  EXACT_MATCH(Generic::wrap(-180,180, 	    -90-789*360),    -90);
  EXACT_MATCH(Generic::crop(-180,180, 	      0+789*360),    180);
  EXACT_MATCH(Generic::crop(-180,180, 	     90-789*360),   -180);
  EXACT_MATCH(Generic::crop(-180,180, 	    450-789*360),   -180);
  EXACT_MATCH(Generic::crop(-180,180, 	    -90-789*360),   -180);

  section("accumulate");
  {
    typedef std::vector<Int*> IPV;
    IPV v;
    v.push_back(new Int(2));
    v.push_back(new Int(4));
    v.push_back(new Int(6));
    FLOAT_MATCH(Generic::accumulate(v.begin(),v.end(),0.0, std::mem_fun(&Int::value)),
		0.0+2.0+4.0+6.0);
  }
  {
    typedef std::vector<Int> VI;
    VI u;
    u.push_back(Int(2));
    u.push_back(Int(4));
    u.push_back(Int(6));
    FLOAT_MATCH(Generic::accumulate(u.begin(),u.end(),1.0, std::mem_fun_ref(&Int::value), std::multiplies<double>()),
		1.0*2.0*4.0*6.0);
  }
  
  section("delete_contents");
  Int::destructions=0;
  typedef std::vector<Int*> IPV;
  IPV v;
  v.push_back(new Int(2));
  v.push_back(new Int(4));
  v.push_back(new Int(6));
  Generic::delete_contents(v);
  EXACT_MATCH(Int::destructions, 3);

  section("nested_delete_contents");
  Int::destructions=0;
  typedef std::vector<IPV*> IPVV;
  IPVV vv;
  {
    IPV* ipvp = new IPV;
    ipvp->push_back(new Int(2));
    ipvp->push_back(new Int(4));
    ipvp->push_back(new Int(6));
    vv.push_back(ipvp);
  }
  {
    IPV* ipvp = new IPV;
    ipvp->push_back(new Int(3));
    ipvp->push_back(new Int(5));
    vv.push_back(ipvp);
  }
  Generic::nested_delete_contents(vv);
  EXACT_MATCH(Int::destructions, 5);

  return count_errors();
}
