/* -*-mode: C++; compile-command: "gmake -C../tests histogram.test"; -*- */
/**
  \file   Tests the corresponding .h file.
  $Header: /u/nn/cvsroot/lissom/tests/histogram.c,v 1.14 2002/08/05 06:33:23 jbednar Exp $
*/
#include "../src/histogram.h"

#include "testutils.h"
#include "pixelutils.h"

#include "../src/histogram.h"
#include "../src/histogramimage.h"

#ifndef NO_MATRIX
#include "../src/matriximagefactory.h"
#include "../src/matrix.h"
#include "../src/matrixhistogram.h"
#endif

#ifdef EXPLICIT_INSTANTIATION_FOR_STATICS
template class Histo::OneDBinList<int,double>;
template class Plot::HSVPixel<Plot::Pixel::FComponent,Plot::Pixel::HueComponent>;
#endif 

int main(int, char**)
{
  section("Sizes");
#ifdef _CRAYMPP
  EXACT_CHECK( int(sizeof(Histo::OneDHistogram<>)), 104);
#else  
  EXACT_CHECK( int(sizeof(Histo::OneDHistogram<>)), 68);
#endif  
  
  section("Construction");
  Histo::OneDHistogram<> h(6);
  Histo::OneDHistogram<> g(6);
  Histo::OneDHistogram<int,int> h_int(1);

  section("Default values");
  FLOAT_MATCH(h.value(0), 0.0);
  FLOAT_MATCH(h.value(1), 0.0);
  FLOAT_MATCH(h.value(2), 0.0);
  FLOAT_MATCH(h.value(3), 0.0);
  FLOAT_MATCH(h.value(4), 0.0);
  FLOAT_MATCH(h.value(5), 0.0);
  
  section("Added values");
  h.add(0,0.0);
  h.add(1,0.1);
  h.add(2,0.2);
  h.add(2,0.2);
  h.add(3,0.3);
  h.add(3,0.3);
  h.add(3,0.3);
  h.add(4,0.4);
  h.add(4,0.4);
  h.add(4,0.4);
  h.add(4,0.4);
  h.add(5,0.5);
  h.add(5,0.5);
  h.add(5,0.5);
  h.add(5,0.5);
  h.add(5,0.5);

  g=h;
  g.add(0,0.0);
  g.add(1,0.1);
  g.add(2,0.2);
  g.add(3,0.3);
  g.add(4,0.4);
  g.add(5,0.5);
  
  FLOAT_MATCH(h.value(0), 0.0);
  FLOAT_MATCH(h.value(1), 0.1);
  FLOAT_MATCH(h.value(2), 0.4);
  FLOAT_MATCH(h.value(3), 0.9);
  FLOAT_MATCH(h.value(4), 1.6);
  FLOAT_MATCH(h.value(5), 2.5);

  EXACT_MATCH(h.count(0), 1);
  EXACT_MATCH(h.count(1), 1);
  EXACT_MATCH(h.count(2), 2);
  EXACT_MATCH(h.count(3), 3);
  EXACT_MATCH(h.count(4), 4);
  EXACT_MATCH(h.count(5), 5);
  
  FLOAT_MATCH(g.value(0), 0.0);
  FLOAT_MATCH(g.value(1), 0.2);
  FLOAT_MATCH(g.value(2), 0.6);
  FLOAT_MATCH(g.value(3), 1.2);
  FLOAT_MATCH(g.value(4), 2.0);
  FLOAT_MATCH(g.value(5), 3.0);

  section("Magnitudes");
  FLOAT_MATCH(h.value_mag(0), 0.0/5.5);
  FLOAT_MATCH(h.value_mag(1), 0.1/5.5);
  FLOAT_MATCH(h.value_mag(2), 0.4/5.5);
  FLOAT_MATCH(h.value_mag(3), 0.9/5.5);
  FLOAT_MATCH(h.value_mag(4), 1.6/5.5);
  FLOAT_MATCH(h.value_mag(5), 2.5/5.5);
  
  FLOAT_MATCH(h.count_mag(0), 1.0/16);
  FLOAT_MATCH(h.count_mag(1), 1.0/16);
  FLOAT_MATCH(h.count_mag(2), 2.0/16);
  FLOAT_MATCH(h.count_mag(3), 3.0/16);
  FLOAT_MATCH(h.count_mag(4), 4.0/16);
  FLOAT_MATCH(h.count_mag(5), 5.0/16);
  
  section("Statistics");
  EXACT_MATCH(int(h.num_bins()),    6);
  FLOAT_MATCH(h.total_value(),      5.5);
  EXACT_MATCH(h.total_count(),     16);
  FLOAT_MATCH(h.max_value(),        2.5);
  FLOAT_MATCH(h.min_value(),        0);
  FLOAT_MATCH(h.max_count(),        5);
  FLOAT_MATCH(h.min_count(),        1);

  FLOAT_MATCH(g.total_value(),      7.0);
  EXACT_MATCH(g.total_count(),     22);

  FLOAT_MATCH(g.sum_value(),        7.0);
  FLOAT_MATCH(g.weighted_sum(),    28.0);
  FLOAT_MATCH(g.weighted_average(),28.0/g.sum_value()/5);

  FLOAT_MATCH(std::arg(g.vector_sum()),     -0.57721*M_PI );
  FLOAT_MATCH(std::abs(g.vector_average()),  std::abs(g.vector_sum())/g.num_bins());
  FLOAT_MATCH(std::arg(g.vector_average()),  std::arg(g.vector_sum()));

  Histo::OneDHistogram<> q(5);
  q.add(3,1);
  
  FLOAT_MATCH(std::abs(q.vector_sum()),       1.0);
  FLOAT_MATCH(std::abs(q.vector_average()),   1.0/5.0);
  FLOAT_MATCH(q.vector_direction(),           3.0/5.0);

  Histo::OneDHistogram<> rr(5);
  rr.add(0,1);
  rr.add(1,1);
  rr.add(2,1);
  rr.add(3,1);
  rr.add(4,1);
  rr.add(5,1);
  FLOAT_MATCH(std::abs(rr.vector_sum()),      0.0);
  FLOAT_MATCH(std::abs(rr.vector_average()),  std::abs(rr.vector_sum())/5.0);

  rr.add(1,2);				      
  FLOAT_MATCH(std::abs(rr.vector_sum()),      2.0);
  FLOAT_MATCH(std::abs(rr.vector_average()),  std::abs(rr.vector_sum())/5.0);
  FLOAT_MATCH(rr.vector_direction(),          1.0/5.0);
  					      
  rr.add(3,2);				      
  FLOAT_MATCH(std::abs(rr.vector_sum()),      2*2*cos(2*M_PI/5));
  FLOAT_MATCH(std::abs(rr.vector_average()),  std::abs(rr.vector_sum())/5.0);
  FLOAT_MATCH(rr.vector_direction(),          2.0/5.0);
  
  section("Peak histogram");
  Histo::OneDHistogram<> p(4);
  p.keep_peak(0,1.0);
  p.keep_peak(2,9.9);
  p.keep_peak(2,3.0);
  p.keep_peak(0,1.0);
  p.keep_peak(1,0.3);
  p.keep_peak(1,84.0);
  p.keep_peak(1,36.0);
  
  FLOAT_MATCH(p.value(0), 1.0);
  FLOAT_MATCH(p.value(1),84.0);
  FLOAT_MATCH(p.value(2), 9.9);
  FLOAT_MATCH(p.value(3), 0.0);

  EXACT_MATCH(p.count(0), 2);
  EXACT_MATCH(p.count(1), 3);
  EXACT_MATCH(p.count(2), 2);
  EXACT_MATCH(p.count(3), 0);
  
  
  section("Subtraction");
  Histo::OneDHistogramDifference<> d(g,h);
  FLOAT_MATCH(d.value(0), 0.0);
  FLOAT_MATCH(d.value(1), 0.1);
  FLOAT_MATCH(d.value(2), 0.2);
  FLOAT_MATCH(d.value(3), 0.3);
  FLOAT_MATCH(d.value(4), 0.4);
  FLOAT_MATCH(d.value(5), 0.5);

  FLOAT_MATCH(d.value_mag(0), d.value(0)/(5.5+7.0));
  FLOAT_MATCH(d.value_mag(1), d.value(1)/(5.5+7.0));
  FLOAT_MATCH(d.value_mag(2), d.value(2)/(5.5+7.0));
  FLOAT_MATCH(d.value_mag(3), d.value(3)/(5.5+7.0));
  FLOAT_MATCH(d.value_mag(4), d.value(4)/(5.5+7.0));
  FLOAT_MATCH(d.value_mag(5), d.value(5)/(5.5+7.0));

  section("Plotting");
  Plot::OneDHistogramImage<> imagev(h,true,10.0,1.0,1.0,3);
  Plot::OneDHistogramImage<> imageh(h,false,10.0,0.25,0.5,0);
  PPM_WRITE("onedhistov",imagev,"Vertical image of a histogram");  
  PPM_WRITE("onedhistoh",imageh,"Horizontal image of a histogram");  

  
#ifndef NO_MATRIX
  section("Matrix Histogram");
  typedef MatrixType<float>::rectangular       Matrix;
  typedef mat::MatrixInterfaceConstant<float>  ConstantMatrix;  
  const Matrix::size_type width=20;
  const Matrix::size_type height=20;
  Matrix lgaussian (height,width);
  Matrix tiltsine  (height,width);
   
  const double sig=width/5;
  const double sig_sq=sig*sig;
  const double PI=3.314159;
  
  for (Matrix::size_type r=0; r<height; r++)
    for (Matrix::size_type c=0; c<width; c++) {
      const double dc=c-width/2.0;
      const double dr=r-height/2.0;
      const double rad_sq=dc*dc+dr*dr;
      
      lgaussian[r][c]   = exp(-rad_sq/(2*PI*sig_sq)); /* Large Gaussian */
      tiltsine[r][c]    = 0.5*sin(r/6.0+c/6.0)+0.5;   /* Tilted sine wave */
    }

  
  ConstantMatrix const_one(1.0,height,width);
  const int num_divisions=24;
  Scale::Linear<double> sinescale(num_divisions);      /* Correct scale for tiltsine's range of 0.0..1.0 */
  Scale::Linear<double> overscale(1.2*num_divisions);  /* Scale intentionally too high; crops or wraps. */
  Histo::OneDHistogram<> mhs = Histo::matrix_histogram(tiltsine, const_one, num_divisions,sinescale,true);
  Histo::OneDHistogram<> mhw = Histo::matrix_histogram(tiltsine, lgaussian, num_divisions,overscale,true);
  Histo::OneDHistogram<> mhc = Histo::matrix_histogram(tiltsine, lgaussian, num_divisions,overscale,false);
  
  section("Matrix Histogram Plotting");
  Matrix one2(height,width); mat::set(one2, 1.0);
  Plot::MatrixImage<>        imagetiltsine(tiltsine,one2,one2,1.0);
  
  Plot::OneDHistogramImage<> imagemhs(mhs,false,10.0,0.25,0.2,1);
  Plot::OneDHistogramImage<> imagemhw(mhw,false,10.0,0.25,0.2,1);
  Plot::OneDHistogramImage<> imagemhc(mhc,false,10.0,0.25,0.2,1);
  
  PPM_WRITE("onedhistomhsource",imagetiltsine,"Matrix used for onedhistomh*");
  PPM_WRITE("onedhistomhs",imagemhs,"Horizontal image of a histogram from a matrix of sine values");
  PPM_WRITE("onedhistomhw",imagemhw,"Horizontal image of a histogram from a matrix, wrapping");
  PPM_WRITE("onedhistomhc",imagemhc,"Horizontal image of a histogram from a matrix, cropping");
#endif
  
  return count_errors();
}
