/* -*-mode: C++; compile-command: "gmake -C../tests image.test"; -*- */
/**
  \file   Tests the corresponding .h file.
  $Header: /u/nn/cvsroot/lissom/tests/image.c,v 1.12 2001/04/08 04:16:44 jbednar Exp $
*/


#include <cmath>
#include "testutils.h"
#include "pixelutils.h"
#include "../src/image.h"
#include "../src/pixel.h"
#include "../src/pnm_io.h"
#include "../src/colorlookup.h"
#include "../src/imagegrid.h"

#ifdef EXPLICIT_INSTANTIATION_FOR_STATICS
template class Plot::HSVPixel<Plot::Pixel::FComponent,Plot::Pixel::HueComponent>;
#endif 


int main(int, char**)
{
  Plot::AARImage<>::default_fg     = Plot::RGBPixel<>(1,1,1,false);
  Plot::AARImage<>::default_bg     = Plot::RGBPixel<>(0.25,0.25,0.25);
  Plot::AARImage<>::default_border = Plot::RGBPixel<>(0.5,0.5,0.5);

  section("Sizes");
#ifdef USE_TNT_MATRIX  
#ifdef _CRAYMPP
  std::cout << "Cray bloats small classes for some reason" << std::endl;
  EXACT_CHECK( int(sizeof(Plot::AARImage<>)), 56);
#else  
  EXACT_CHECK( int(sizeof(Plot::AARImage<>)), 28);
#endif  
#else
#ifdef _CRAYMPP
  std::cout << "Cray bloats small classes for some reason" << std::endl;
  EXACT_CHECK( int(sizeof(Plot::AARImage<>)), 80);
#else  
  EXACT_CHECK( int(sizeof(Plot::AARImage<>)), 40);
#endif  
#endif
  
  section("Construction");
  Plot::AARImage<> i(20,30);

  section("Drawing lines, dots, and rectangles");
  i.draw_line( 1, 1,  3, 15,   Plot::RGBPixel<>(0.2,0.9,0.9));
  i.draw_line( 1, 1, 15,  3,   Plot::RGBPixel<>(0.9,0.2,0.2));
  i.draw_line( 5, 8, 15,  3,   Plot::RGBPixel<>(0.4,0.2,0.2));
  i.draw_line( 5, 8,  3, 15,   Plot::RGBPixel<>(0.2,0.4,0.4));
  i.draw_line( 3, 3,  3,  3,   Plot::RGBPixel<>(0.6,0.6,0.2));

  i.draw_pixel(    2, 2,       Plot::RGBPixel<>(0.9,0.9,0.2));
  i.draw_rectangle(5, 2, 7, 9, Plot::RGBPixel<>(0.3,0.3,0.1));

  section("Drawing bars");
  i.draw_bar(24, 4,   M_PI/8,9, 3, Plot::HSVPixel<>(0.1,0.8,1.0));
  i.draw_bar(24,13, 3*M_PI/8,9, 2, Plot::HSVPixel<>(0.3,0.8,1.0));
  for (float f=0; f<M_PI; f+=M_PI/5)
    i.draw_bar(13,7, f, 11, 1, Plot::HSVPixel<>(f/M_PI,0.5,1.0));

  i.draw_bar(2,2,      0, 2);
  i.draw_bar(2,2, M_PI/2, 2);
  
  section("ImageGrid");
  typedef MatrixType<Plot::AARImage<> >::rectangular MatrixOfImages;
  MatrixOfImages moi(2,3);
  
  moi[0][0]=i;  moi[0][1]=i;  moi[0][2]=i;
  moi[1][0]=i;  moi[1][1]=i;  moi[1][2]=i;
  Plot::ImageGrid<> grid(moi);
  
  section("ImagePointerGrid");
  typedef MatrixType<Plot::AARImage<>* >::rectangular MatrixOfImagePointers;
  MatrixOfImagePointers moip(2,3);
  
  moip[0][0]=&i;  moip[0][1]=&i;  moip[0][2]=&i;
  moip[1][0]=&i;  moip[1][1]=&i;  //moip[1][2]=&i;
  Plot::ImagePointerGrid<> grid2(moip);
  
  section("Image file output");
  PPM_WRITE("image",i,"Simple line drawing");
  PPM_WRITE("grid",grid,"3x2 grid of image.ppm");
  PPM_WRITE("grid2",grid2,"3x2 grid of pointers to image.ppm");
  
  return count_errors();
}







