/* -*-mode: C++; compile-command: "gmake -C../tests matrixadapter.test"; -*- */
/**
  \file   Tests the corresponding .h file.
  $Header: /u/nn/cvsroot/lissom/tests/matrixadapter.c,v 1.8 2001/04/08 04:16:44 jbednar Exp $
*/

#include "testutils.h"
#include "pixelutils.h"
#include "../src/matriximage.h"
#include "../src/pixel.h"
#include "../src/pnm_io.h"
#include "../src/colorlookup.h"
#include "../src/matrixadapter.h"

#ifdef EXPLICIT_INSTANTIATION_FOR_STATICS
template class Plot::HSVPixel<Plot::Pixel::FComponent,Plot::Pixel::HueComponent>;
#endif 


int main(int, char**)
{
  section("Value, dimensions, and offset scaling");
  typedef MatrixType<float>::rectangular Matrix;
  Matrix g(10,20);
  for (Matrix::size_type r=0; r<g.nrows(); r++)
    for (Matrix::size_type c=0; c<g.ncols(); c++)
      g[r][c]=sin(r+c);
  Plot::MatrixImage<> i(g,1.0,0,Plot::RYWColorLookup<>() );
  //
  mat::MatrixAdapter<Matrix> f(g);
  FLOAT_MATCH( mat::elem(f,1,1), mat::elem(g,1,1) );
  FLOAT_MATCH( mat::elem(f,1,2), mat::elem(g,1,2) );
  //
  double   mult=0.8;
  double   off=0.5;
  int      height=g.nrows()*2;
  int      width=g.ncols()/2;
  int      originr=1;
  int      originc=1;
  // Avoid large values (over 2.0) since the checking formulas below do not respect bounds
  double   sizemult=1.75;
  const Scale::Linear<float> scale(mult,off);
  typedef Boundary::AAArbitraryRectangle<double,int> Region;  
  const Region viewport(originc,originr,originc-width,originr-height);
  mat::MatrixAdapter<Matrix> h(g,scale,viewport);
  // Note that the () indexes differ between TNT and MTL 
  FLOAT_MATCH( h(2,2), g(2-originr,2-originc)*mult+off );
  FLOAT_MATCH( h(2,3), g(2-originr,3-originc)*mult+off );
  FLOAT_MATCH( mat::elem(h,2,2), mat::elem(g,2-originr,2-originc)*mult+off );
  FLOAT_MATCH( mat::elem(h,2,3), mat::elem(g,2-originr,3-originc)*mult+off );

#ifndef USE_TNT_MATRIX
  section("Operator() vs. mat::elem()");
  // These two ways of specifying an element should refer to the same element under MTL, but not TNT
  FLOAT_MATCH( mat::elem(h,2,2), h(2,2));
  FLOAT_MATCH( mat::elem(h,2,3), h(2,3));
#endif 

  section("Size scaling");
  const Region viewport2(originc,originr,originc-width,originr-height);
  mat::MatrixAdapter<Matrix> ss(g,scale,viewport2,sizemult);
  FLOAT_MATCH( mat::elem(ss,2,2), mat::elem(g,int(2/sizemult)-originr,int(2/sizemult)-originc)*mult+off );
  FLOAT_MATCH( mat::elem(ss,2,3), mat::elem(g,int(2/sizemult)-originr,int(3/sizemult)-originc)*mult+off );

  section("Wrapping an int matrix as a double using MatrixInterface");
  typedef MatrixType<int>::rectangular IntMatrix;
  IntMatrix j(10,5);
  {
    for (IntMatrix::size_type r=0; r<j.nrows(); r++)
      for (IntMatrix::size_type c=0; c<j.ncols(); c++)
	j[r][c]=r+c;
  }
  EXACT_MATCH( mat::elem(j,2,2), 2+2);
  EXACT_MATCH( mat::elem(j,2,3), 2+3);
  
  double imult=0.125;
  const Scale::Linear<double> iscale(imult);
  mat::MatrixInterfaceAdapter<IntMatrix,double> ja(j,iscale);
  FLOAT_MATCH( mat::elem(ja,2,2), mat::elem(j,2,2)*imult );
  FLOAT_MATCH( mat::elem(ja,2,3), mat::elem(j,2,3)*imult );

  section("Image file output");
  Plot::MatrixImage<> image1(ss,1.0,0,Plot::ValueColorLookup<>() );
  Plot::MatrixImage<> image2(ja);
  PPM_WRITE("bordered",image1,"Border added");
  PPM_WRITE("i2d",image2,"Integer array viewed as double");
  
  return count_errors();
}







