/* -*-mode: C++; compile-command: "gmake -C../tests matriximage.test"; -*- */
/**
  \file   Tests the corresponding .h file.
  $Header: /u/nn/cvsroot/lissom/tests/matriximage.c,v 1.13 2001/11/25 23:48:29 jbednar Exp $
*/

#include "testutils.h"
#include "pixelutils.h"
#include "../src/matriximage.h"
#include "../src/pixel.h"
#include "../src/pnm_io.h"
#include "../src/colorlookup.h"
#include "../src/imagegrid.h"

#ifdef EXPLICIT_INSTANTIATION_FOR_STATICS
template class Plot::HSVPixel<Plot::Pixel::FComponent,Plot::Pixel::HueComponent>;
#endif 

int main(int, char**)
{
  Plot::AARImage<>::default_fg     = Plot::RGBPixel<>(1,1,1,false);
  Plot::AARImage<>::default_bg     = Plot::RGBPixel<>(0.25,0.25,0.25);
  Plot::AARImage<>::default_border = Plot::RGBPixel<>(0.5,0.5,0.5);
  
  section("Sizes");
#ifdef USE_TNT_MATRIX  
#ifdef _CRAYMPP
  std::cout << "Cray bloats small classes for some reason" << std::endl;
  EXACT_CHECK( int(sizeof(Plot::MatrixImage<>)), 88);
#else
  EXACT_CHECK( int(sizeof(Plot::MatrixImage<>)), 48);
#endif
#else
#ifdef _CRAYMPP
  std::cout << "Cray bloats small classes for some reason" << std::endl;
  EXACT_CHECK( int(sizeof(Plot::MatrixImage<>)), 112);
#else
  EXACT_CHECK( int(sizeof(Plot::MatrixImage<>)), 60);
#endif
#endif
  
  section("Construction");
  typedef MatrixType<float>::rectangular Matrix;
  const Matrix::size_type height=10;
  const Matrix::size_type width=20;
  Matrix g(height,width);
  for (Matrix::size_type r=0; r<g.nrows(); r++)
    for (Matrix::size_type c=0; c<g.ncols(); c++)
      g[r][c]=sin(r+c);
  Plot::MatrixImage<> i(g,1.0,0,Plot::RYWColorLookup<>() );
  //
  Matrix h;
  pgm_read("input/feep.pgm",h);
  Plot::MatrixImage<> f(h,1.0,0,Plot::ValueColorLookup<>() );
  //
  Matrix zero(7,25); mat::set(zero,0.0);
  Matrix one(7,25);  mat::set(one, 1.0);
  Plot::MatrixImage<> hsv(zero,one,h,3.0);  /* Scale is larger than others */
  hsv.draw_pixel(6,0,Plot::RGBPixel<>(0.2,0.2,0.9));
  hsv.draw_bar_on_matrix_element(3,10, M_PI/4, 6, 1, Plot::HSVPixel<>(0.1,0.8,1.0));
  
  typedef MatrixType<Plot::MatrixImage<> >::rectangular MatrixOfImages;
  MatrixOfImages moi(2,3);

  moi[0][0]=i;    moi[0][1]=h;  
  moi[1][0]=hsv;  moi[1][1]=f;  moi[1][2]=g;  
  Plot::ImageGrid<> stack(moi,2,4);
  
  section("Image file output");
  PPM_WRITE("ryw",        i,"Constructed from RYW color table");
  PPM_WRITE("value",      f,"Constructed from ValueColorLookup");
  PPM_WRITE("hsv",      hsv,"Constructed from HSV");
  PPM_WRITE("stack",  stack,"Constructed from a matrix of images");

  return count_errors();
}







