/** \file
  Message handlers that count the number of calls and keep the
  message but do not display it.

  $Header: /u/nn/cvsroot/lissom/tests/msgutils.h,v 1.8 2001/11/24 22:17:10 jbednar Exp $
*/

#ifndef __MSGUTILS_H__
#define __MSGUTILS_H__

#include "testutils.h"
#include "../src/msg.h"

namespace Msg {

int notifies=0;  string last_notify="";
int   errors=0;  string last_error="";
int warnings=0;  string last_warning="";

string messages="";

inline  void countnotify  (const string& s, const bool terminate=true )
{  notifies++; last_notify=s;   messages +=               s+(terminate? "\n" : ""); }

inline  void countwarning (const string& s, const bool terminate=true )
{  warnings++; last_warning=s;  messages += "Warning: " + s+(terminate? "\n" : ""); }

inline  void counterror   (const string& s, const bool terminate=true )
{  errors++;   last_error=s;    messages += "Error: "   + s+(terminate? "\n" : ""); }


/** Wrapper for the above three functions */
LevelHandler<> counting_message_handler =
LevelHandler<>().register_handler(Notify,countnotify).register_handler(Warning,countwarning).register_handler(Error,counterror);

/** Check for errors and warnings, announcing them and then wiping the
   slate clean if any are found. */
inline void check_for_unexpected_messages()
{
  if (errors!=0) {
    num_errors(true);
    std::cerr << "Error: " << errors << " unexpected error message(s) found; last was: " << last_error << std::endl;
    errors=0;
    last_error="";
  }
  
  if (warnings!=0) {
    num_errors(true);
    std::cerr << "Error: " << errors << " unexpected warning message(s) found; last was: " << last_warning << std::endl;
    warnings=0;
    last_warning="";
  }
}

/** Consumes an expected error, ensuring that it appears as expected.
    If warnonly does not report a mismatch as an error. */
inline void expected_error(string msg,int num=1,bool warnonly=false)
{
  if (warnonly) {
    EXACT_CHECK( errors,         num);
    EXACT_CHECK( last_error,     msg);
  }
  else {
    EXACT_MATCH( errors,         num);
    EXACT_MATCH( last_error,     msg);
  }
  
  errors=0;
  last_error="";
}

 
/** Consumes an expected error, ensuring that it appears as expected */
inline void expected_warning(string msg,int num=1)
{
  EXACT_MATCH( warnings,       num);
  EXACT_MATCH( last_warning,   msg);
  warnings=0;
  last_warning="";
}

 
} /* namespace Msg */


#endif /* __MSGUTILS_H__ */


