/* -*-mode: C++; compile-command: "cd ../tests ; rm -f parameter ; g++ -g -Wall parameter.c -o parameter; ./parameter"; -*- */
/**
  \file   Tests parameter.h
  $Header: /u/nn/cvsroot/lissom/tests/parameter.c,v 1.29 2001/04/08 04:16:44 jbednar Exp $
*/

#include "testutils.h"
#include "msgutils.h"

#include "../src/parameter.h"
#include "../src/polymorph.h"
#include "../src/stringparser.h"

#ifdef EXPLICIT_INSTANTIATION_FOR_STATICS
template class Msg::LevelHandler<Msg::MessageLevel,Msg::NumMessageLevels>;
template class PolymorphBase<Tristate>;
template class PolymorphBase<int>;
template class PolymorphBase<double>;
template class PolymorphBase<string>;
template class ParameterDefinition<int>;
#endif 



/* Constants */
const double init_f=0.5;



/* Sample parameters */
Tristate bparam=True;
double fParam=init_f;
int    i_param=6;
int    i_param2=9;
string usparam("initialstring");



/* Sample setfn */
int setfn_called=0;
SETFN_DECLARE2(i_param_setfn2);
cmdstat i_param_setfn2(const string& name, Typeless& param, const Typeless& newval)
{
  (void)name;
  param.set(newval);

  setfn_called++;
  return 0;
}



/* Useful macros */
#define PARAM(type,name) \
ParamType param_ ## name = ParamType(make_Polymorph(&name),#name)
//ParamType param_ ## name = ParamType(#name,type,false,(void*)&name)
#define PARAM_N(type,name,doc) \
PARAM(type,name).add_doc(doc)
#define PARAM_I(type,name,lower,upper,doc) \
PARAM_N(type,name,doc).add_lower_bound(lower).add_upper_bound(upper)


#undef SETFNOBJ_DEFINE2
#define SETFNOBJ_DEFINE2(cargs,paramname,fnname) \
param_ ## paramname.add_setfn(new setfnobj2_ ## fnname(cargs))

#undef SETFN_DEFINE2
#define SETFN_DEFINE2(paramname,fnname) SETFNOBJ_DEFINE2(,paramname,fnname)

int main(int, char**)
{
  /* For pretty */
  Polymorph<Tristate>::set_typestring("Boolean");
  Polymorph<int>::set_typestring("Integer");
  Polymorph<double>::set_typestring("Float");
  Polymorph<string>::set_typestring("String");


  section("Construction");
  typedef ParameterDefinition<>        ParamType;
  Msg::LevelHandler<>::set_default_instance(&Msg::counting_message_handler);
  
  section("Parameter definition");
  PARAM_I(PARAM_3BOOL, bparam, Uninitialized,True, "The documentation for %s,\nwhich takes two lines.");
  PARAM_I(PARAM_DOUBLE,fParam, 0,            1,    "The documentation for %s.\n");
  PARAM_N(PARAM_USTRING,usparam,                   "The\nthree\nline documentation for %s.");
  PARAM_I(PARAM_INT,   i_param,0,            12,   "Documentation for %s.");
  param_fParam.add_default_expr("0.01*39");
  SETFN_DEFINE2(i_param,i_param_setfn2);
  PARAM(  PARAM_INT,   i_param2);
  Msg::check_for_unexpected_messages();


  section("Member functions");
  EXACT_MATCH( param_bparam.usage_string(),        "bparam=True                            (Boolean with range [Uninitialized,True])");
  EXACT_MATCH( param_i_param.doc_string(),         "i_param=6                              (Integer with range [0,12])\n\nDocumentation for i_param.\n");
  EXACT_MATCH( string(param_fParam.get_default_expr()), string("0.01*39"));
  EXACT_MATCH( param_fParam.stringrep(param_fParam.datavalue()),          "0.5");
  EXACT_MATCH( param_fParam.stringrep(param_fParam.datavalue(),"5"),      "  0.5");
  EXACT_MATCH( param_fParam.stringrep(param_fParam.datavalue(),"05"),     "000.5");
  EXACT_MATCH( param_fParam.stringrep(param_fParam.datavalue(),"06.2"),   "000.50");
  EXACT_MATCH( param_fParam.stringrep(param_fParam.datavalue(),"6.2"),    "  0.50");
  EXACT_MATCH( param_fParam.stringrep(param_fParam.datavalue(),"6.0"),    "     0");
  EXACT_MATCH( param_i_param.stringrep(param_i_param.datavalue(),"05"),   "00006");
  EXACT_MATCH( param_i_param2.stringrep(param_i_param2.datavalue()),      "9");
  EXACT_MATCH( param_usparam.name(),                     "usparam");
  EXACT_MATCH( (param_usparam<param_fParam),             false);
  EXACT_MATCH( param_usparam.stringrep(param_usparam.datavalue(),"",true), "\"initialstring\"");
  string other("other");
  EXACT_MATCH( param_usparam.stringrep(Polymorph<string>(other)),          "other");
  Msg::check_for_unexpected_messages();


  section("Setting");
  StringParser sp;
  EXACT_MATCH( param_fParam.name(),                  "fParam");
  EXACT_MATCH( param_fParam.datavalue().isnumeric(), true);
  EXACT_MATCH( param_fParam.set(sp,"0.123"),         true);    FLOAT_MATCH( fParam,  0.123);
  EXACT_MATCH( param_fParam.datavalue().stringrep(), "0.123");
  EXACT_MATCH( param_i_param.set(sp,"9"),            true);    EXACT_MATCH( i_param, 9);
  EXACT_MATCH( param_usparam.set(sp,"secondstring"), true);    EXACT_MATCH( usparam, string("secondstring"));
  EXACT_MATCH( param_bparam.set(sp,"True"),          true);    EXACT_MATCH( bparam, True);
  EXACT_MATCH( param_bparam.set(sp,"False"),         true);    EXACT_MATCH( bparam, False);
  EXACT_MATCH( param_bparam.set(sp,"Uninitialized"), true);    EXACT_MATCH( bparam, Uninitialized);
  EXACT_MATCH( param_bparam.datavalue().stringrep(), "Uninitialized");
  EXACT_MATCH( param_bparam.stringrep(),             "Uninitialized");
  Msg::check_for_unexpected_messages();

  section("Error handling");
  EXACT_MATCH( param_bparam.check2(),     0);
  Msg::check_for_unexpected_messages();   
  EXACT_MATCH( param_fParam.check2(),     0);
  EXACT_MATCH( param_i_param.check2(),    0);
  EXACT_MATCH( param_i_param.check2(),    0);
  EXACT_MATCH( param_usparam.check2(),    0);
  Msg::check_for_unexpected_messages();  
  
  int    largei=300;
  int    smalli=-5;
  double largef=1.1;
  double smallf=-0.5;
  string longstring("01234567890123456789012345678901234567890123456789");


  section("Typeless/Polymorph implementation tests");
  Msg::check_for_unexpected_messages();
  Polymorph<Tristate>  pb(bparam);
  Polymorph<int>       pi(i_param);
  Polymorph<double>    pf(fParam);

  EXACT_MATCH( param_bparam.check2(),     0);
  EXACT_MATCH( param_fParam.check2(),     0);
  EXACT_MATCH( param_i_param.check2(),    0);
  EXACT_MATCH( param_i_param2.check2(),   0);
  Msg::check_for_unexpected_messages();
  
  EXACT_MATCH( param_bparam.check2(pb),   0);
  EXACT_MATCH( param_fParam.check2(pf),   0);
  EXACT_MATCH( param_i_param.check2(pi),  0);
  EXACT_MATCH( param_i_param2.check2(pi), 0);
  Msg::check_for_unexpected_messages();
  
  Polymorph<int>    p_largei(largei);
  Polymorph<int>    p_smalli(smalli);
  Polymorph<double> p_largef(largef);
  Polymorph<double> p_smallf(smallf);
  
  EXACT_MATCH( param_bparam.check2(p_smalli),    1);  Msg::expected_error("bparam too small (-5 < Uninitialized)");
  EXACT_MATCH( param_bparam.check2(p_largei),    1);  Msg::expected_error("bparam too large (300 > True)");
  EXACT_MATCH( param_fParam.check2(p_smallf),    1);  Msg::expected_error("fParam too small (-0.5 < 0)");
  EXACT_MATCH( param_fParam.check2(p_largef),    1);  Msg::expected_error("fParam too large (1.1 > 1)");
  EXACT_MATCH( param_i_param.check2(p_smalli),   1);  Msg::expected_error("i_param too small (-5 < 0)");
  EXACT_MATCH( param_i_param.check2(p_largei),   1);  Msg::expected_error("i_param too large (300 > 12)");
  EXACT_MATCH( param_i_param2.check2(p_smalli),  0);
  EXACT_MATCH( param_i_param2.check2(p_largei),  0);
  Msg::check_for_unexpected_messages();

  EXACT_MATCH( param_bparam.set(sp,"-5"),    false);  Msg::expected_error("bparam too small (-5 < Uninitialized)");
  EXACT_MATCH( param_bparam.set(sp,"300"),   false);  Msg::expected_error("bparam too large (300 > True)");
  EXACT_MATCH( param_fParam.set(sp,"-0.5"),  false);  Msg::expected_error("fParam too small (-0.5 < 0)");
  EXACT_MATCH( param_fParam.set(sp,"1.1"),   false);  Msg::expected_error("fParam too large (1.1 > 1)");
  EXACT_MATCH( param_i_param.set(sp,"-5"),   false);  Msg::expected_error("i_param too small (-5 < 0)");
  EXACT_MATCH( param_i_param.set(sp,"300"),  false);  Msg::expected_error("i_param too large (300 > 12)");
  EXACT_MATCH( param_i_param2.set(sp,"-5"),  true);
  EXACT_MATCH( param_i_param2.set(sp,"300"), true);  

  return count_errors();
}
