/* -*-mode: C++; compile-command: "gmake -C../tests pixel.test"; -*- */
/**
  \file   Tests the corresponding .h file.
  $Header: /u/nn/cvsroot/lissom/tests/pixel.c,v 1.9 2001/04/08 04:16:44 jbednar Exp $
*/

#include "pixelutils.h"

#include "../src/boundednumber.h"
#include "../src/pixel.h"

#ifdef EXPLICIT_INSTANTIATION_FOR_STATICS
template class Plot::HSVPixel<Plot::Pixel::FComponent,Plot::Pixel::HueComponent>;
#endif 

int main(int, char**)
{
  section("Sizes");
#ifdef _CRAYMPP
  std::cout << "Cray bloats small classes for some reason" << std::endl;
  EXACT_CHECK( int(sizeof(Plot::Pixel)),      8);
  EXACT_CHECK( int(sizeof(Plot::RGBPixel<>)), 32);
  EXACT_CHECK( int(sizeof(Plot::HSVPixel<>)), 32);
#else
  EXACT_CHECK( int(sizeof(Plot::Pixel)),      1);
  EXACT_CHECK( int(sizeof(Plot::RGBPixel<>)), 4);
  EXACT_CHECK( int(sizeof(Plot::HSVPixel<>)), 16);
#endif
  
  section("Construction");
  Plot::RGBPixel<> r(0.5,0.1,1.0);        RGB_MATCH(r,0.5,0.1,1.0);
  Plot::HSVPixel<> h(0.5,0.1,1.0);        HSV_MATCH(h,0.5,0.1,1.0);

  section("Cropping construction");
  Plot::RGBPixel<> crop_r (-1.2,1.1,3002.0);  RGB_MATCH(crop_r,0.0,1.0,1.0);
  Plot::HSVPixel<> crop_h (-1.2,1.1,3002.0);  HSV_MATCH(crop_h,0.8,1.0,1.0);
  Plot::HSVPixel<> crop_h2(123.2,1.1,3002.0); HSV_MATCH(crop_h2,0.2,1.0,1.0);

  section("Copy construction");
  Plot::RGBPixel<> rr(r);           RGB_MATCH(rr,r.red(),r.green(),r.blue());
  Plot::HSVPixel<> hh(h);           HSV_MATCH(hh,h.hue(),h.saturation(),h.value());

  // Desired values computed from the GIMP's color selector
  section("Copy conversion");
  Plot::RGBPixel<> rh(h);           RGB_MATCH(rh,229.0/255.0,1.0,1.0);
  Plot::HSVPixel<> hr(r);           HSV_LOOSE_MATCH(hr,266.0/360.0,90.0/100.0,1.0);

  section("Arithmetic");
  const double m=0.57;       rr*=m;
  RGB_MATCH(rr,   r.red()*m,        r.green()*m,           r.blue()*m );
  RGB_MATCH(rr+r, r.red()+rr.red(), r.green()+rr.green(),  r.blue()+rr.blue() );
  
  section("Cropping arithmetic");
  const double m2=100000.57;  rr*=m2;   RGB_MATCH(rr,   1.0,1.0,1.0);
					  RGB_MATCH(rr+r, 1.0,1.0,1.0);
  const double m3=-100000.57; rr*=m3;   RGB_MATCH(rr,   0.0,0.0,0.0);
  rr=r; rr*=-2.4;                       RGB_MATCH(rr,   0.0,0.0,0.0);
  
  section("Binary file output");
  std::ofstream os("binaryoutput");
  r.write_binary(os,(unsigned char)(255));

  return count_errors();
}
