/** \file
  Routines for testing the functionality and correctness of files using Pixels.

  $Header: /u/nn/cvsroot/lissom/tests/pixelutils.h,v 1.7 2001/04/24 02:05:03 jbednar Exp $
*/

#ifndef __PIXELUTILS_H__
#define __PIXELUTILS_H__

#include "testutils.h"
#include "../src/pixel.h"

#define CROP10(val) Generic::crop(0.0,1.0,(val))

/** Compares the given RGB pixel against the given values,
    using a loose tolerance since the type is coarse-grained. */
#define RGB_MATCH(pixel,R,G,B)                        \
  LOOSE_MATCH((pixel).red(),        CROP10(R), 0.02); \
  LOOSE_MATCH((pixel).green(),      CROP10(G), 0.02); \
  LOOSE_MATCH((pixel).blue(),       CROP10(B), 0.02)

/** Compares the given HSV pixel against the given values. */
#define HSV_MATCH(pixel,H,S,V)                  \
  FLOAT_MATCH((pixel).hue(),        CROP10(H)); \
  FLOAT_MATCH((pixel).saturation(), CROP10(S)); \
  FLOAT_MATCH((pixel).value(),      CROP10(V))

/** Compares the given RGB pixel against the given values,
    using a loose tolerance appropriate for values converted from RGB. */
#define HSV_LOOSE_MATCH(pixel,H,S,V)                  \
  LOOSE_MATCH((pixel).hue(),        CROP10(H), 0.01); \
  LOOSE_MATCH((pixel).saturation(), CROP10(S), 0.01); \
  LOOSE_MATCH((pixel).value(),      CROP10(V), 0.01)

/** Writes the given AARImage variable to the given output file */
#ifdef ERRORS_ONLY
#define PPM_WRITE(filenamebase,varname,comment) \
  {  std::ofstream os("output/" filenamebase ".ppm"); \
     varname.ppm_write(os,comment,(unsigned char)(255)); }
#else
#define PPM_WRITE(filenamebase,varname,comment) \
  {  std::ofstream os("output/" filenamebase ".ppm"); \
     varname.ppm_write(os,comment,(unsigned char)(255)); }\
  std::cout << "       " "output/" filenamebase ".ppm:" "\t" << comment << std::endl
#endif

#endif /* __PIXELUTILS_H__ */
