/* -*-mode: C++; compile-command: "cd ../tests ; rm -f polymorph ; g++ -g -Wall polymorph.c -o polymorph; ./polymorph"; -*- */
/**
  \file   Tests polymorph.h
  $Header: /u/nn/cvsroot/lissom/tests/polymorph.c,v 1.4 2001/04/08 04:16:45 jbednar Exp $
*/

#include "testutils.h"

#include "../src/polymorph.h"

#ifdef EXPLICIT_INSTANTIATION_FOR_STATICS
template class PolymorphBase<Tristate>;
template class PolymorphBase<int>;
template class PolymorphBase<double>;
template class PolymorphBase<string>;
#endif 


int      i=5;
double   f=5.2;
string   s="cheese";
Tristate t=Uninitialized;
int main(int, char**)
{
  /* For pretty */
  Polymorph<Tristate>::set_typestring("Boolean");
  Polymorph<int>::set_typestring("Integer");
  Polymorph<double>::set_typestring("Float");
  Polymorph<string>::set_typestring("String");
  

  section("Construction");
  /* Names: p<datatype><p|l> (p for pointer, l for literals) */
  Polymorph<int>    pip(&i);
  Polymorph<int>    pil(7);
  Polymorph<double> pfp(&f);
  Polymorph<double> pfl(3.14);
  Polymorph<string> psp(&s);
  Polymorph<string> psl("bit soup");
  Polymorph<Tristate> ptp(&t);
  Polymorph<Tristate> ptl(False);


  section("Comparison");
  EXACT_MATCH(     (pip<pil),           true);
  EXACT_MATCH(     (pip<2),             false);
  EXACT_MATCH(     (pfp<pfl),           false);
  EXACT_MATCH(     (pfp<99.99),         true);
  EXACT_MATCH(     (psp<psl),           false);
  EXACT_MATCH(     (psp<"ghee"),        true);
  EXACT_MATCH(     (ptp<ptl),           true);
  EXACT_MATCH(     (ptl<True),          true);

  /* This section could be moved to a test file for stringutils.h */
  section("Generic string representations");
  using String::stringrep;
  using String::StreamFormat;
  const int    i1=6;
  const double f1=0.5;
  const double f2=0.54321;
  const char*  c1="romeo";
  EXACT_MATCH( stringrep(i1),                           string("6"       ));
  EXACT_MATCH( stringrep(i1,StreamFormat(5,'0')),       string("00006"   ));
  EXACT_MATCH( stringrep(f1,StreamFormat()),            string("0.5"     ));
  EXACT_MATCH( stringrep(f1,StreamFormat(6,'0')),       string("0000.5"  ));
  EXACT_MATCH( stringrep(f1,StreamFormat(6)),           string("   0.5"  ));
  EXACT_MATCH( stringrep(f2),                           string("0.54321" ));
  EXACT_MATCH( stringrep(f2,StreamFormat(-1,' ',1)),    string("0.5"     ));
  EXACT_MATCH( stringrep(f1,StreamFormat(6,'0',3)),     string("00.500"  ));
  EXACT_MATCH( stringrep(f2,StreamFormat(6,'0',3)),     string("00.543"  ));
  EXACT_MATCH( stringrep(f2,StreamFormat(6,' ',3)),     string(" 0.543"  ));
  EXACT_MATCH( stringrep(f2,StreamFormat(6,'0',1)),     string("0000.5"  ));
  EXACT_MATCH( stringrep(f2,StreamFormat(6,' ',1)),     string("   0.5"  ));
  EXACT_MATCH( stringrep(c1),                           string("romeo"   ));
  EXACT_MATCH( stringrep(c1,StreamFormat(2)),           string("romeo"   ));
  EXACT_MATCH( stringrep(c1,StreamFormat(8)),           string("   romeo"));
  EXACT_MATCH( stringrep(c1,StreamFormat(8,'_')),       string("___romeo"));

  
  section("String Representation");
  EXACT_MATCH(     pip.stringrep(),     		string("5"));
  EXACT_MATCH(     pip.stringrep(StreamFormat(3)),    	string("  5"));
  EXACT_MATCH(     pip.stringrep(StreamFormat(3,'0')),	string("005"));
  EXACT_MATCH(     pip.stringrep(StreamFormat(3,'_')),	string("__5"));
  EXACT_MATCH(     pil.stringrep(),     		string("7"));
  EXACT_MATCH(     pfp.stringrep(),     		string("5.2"));
  EXACT_MATCH(     pfp.stringrep(StreamFormat(2)),    	string("5.2"));
  EXACT_MATCH(     pfp.stringrep(StreamFormat(6)),    	string("   5.2"));
  EXACT_MATCH(     pfp.stringrep(StreamFormat(6,'0')),	string("0005.2"));
  EXACT_MATCH(     pfp.stringrep(StreamFormat(6,'0',2)),string("005.20"));
  EXACT_MATCH(     pfp.stringrep(StreamFormat(6,' ',2)),string("  5.20"));
  EXACT_MATCH(     pfl.stringrep(),     		string("3.14"));
  EXACT_MATCH(     psp.stringrep(),     		string("cheese"));
  EXACT_MATCH(     psp.stringrep(StreamFormat(2)),    	string("cheese"));
  /* Disabled because g++ 2.95 and earlier appear to ignore
     width specifications for stream output of strings */
  //EXACT_MATCH(   psp.stringrep(8) ,   string("  cheese"));
  //EXACT_MATCH(   psp.stringrep(8,'_'),string("__cheese"));
  EXACT_MATCH(     psl.stringrep(),     string("bit soup"));
  EXACT_MATCH(     ptp.stringrep(),     string("Uninitialized"));
  EXACT_MATCH(     ptl.stringrep(),     string("False"));

  
  section("Type string");
  EXACT_MATCH(     pip.typestring(),    string("Integer"));
  EXACT_MATCH(     pil.typestring(),    string("Integer"));
  EXACT_MATCH(     pfp.typestring(),    string("Float"));
  EXACT_MATCH(     pfl.typestring(),    string("Float"));
  EXACT_MATCH(     psp.typestring(),    string("String"));
  EXACT_MATCH(     psl.typestring(),    string("String"));
  EXACT_MATCH(     ptp.typestring(),    string("Boolean"));
  EXACT_MATCH(     ptl.typestring(),    string("Boolean"));

  
  section("Conversion to number");
  EXACT_MATCH(pip.isnumeric(),         	true);
  EXACT_MATCH(pil.isnumeric(),         	true);
  EXACT_MATCH(pfp.isnumeric(),         	true);
  EXACT_MATCH(pfl.isnumeric(),         	true);
  EXACT_MATCH(ptp.isnumeric(),         	true);
  EXACT_MATCH(ptl.isnumeric(),         	true);
  EXACT_MATCH(psp.isnumeric(),         	false);
  EXACT_MATCH(psl.isnumeric(),         	false);
  FLOAT_MATCH(double(pip.numericvalue()),double(5.0));
  FLOAT_MATCH(double(pfp.numericvalue()),double(5.2));
  FLOAT_MATCH(double(ptp.numericvalue()),double(-1.0));
  FLOAT_MATCH(double(psp.numericvalue()),double(0.0)); // Should this be an error?

  
  section("Cloning");
  Typeless* pipc = pip.clone();
  FLOAT_MATCH(double(pipc->numericvalue()),double(5.0));
  delete pipc;


  section("Setting values");
  pip.set(pil);
  EXACT_MATCH(pip.stringrep(),     pil.stringrep());    Polymorph<int>   pil2(2);  
  EXACT_MATCH(pil2.stringrep(),    string("2"));        pip.set(pil2);
  EXACT_MATCH(pip.stringrep(),     pil2.stringrep());   Typeless& pilr = pil; pip.set(pil);
  EXACT_MATCH(pip.stringrep(),     pilr.stringrep());   pfp.set(pfl);
  EXACT_MATCH(pfp.stringrep(),     pfl.stringrep());    psl.set(psp);
  EXACT_MATCH(psl.stringrep(),     psp.stringrep());    ptp.set(ptl);
  EXACT_MATCH(ptp.stringrep(),     ptl.stringrep());
  
  return count_errors();
}
