/* -*-mode: C++; compile-command: "gmake -C../tests valuegen.test"; -*- */
/**
  \file   Tests the corresponding .h file.
  $Header: /u/nn/cvsroot/lissom/tests/valuegen.c,v 1.16 2001/04/08 04:16:45 jbednar Exp $
*/

#include "testutils.h"
#include "../src/valuegen.h"

#ifdef EXPLICIT_INSTANTIATION_FOR_STATICS
template ShuffledRandom<double>;
template Msg::LevelHandler<Msg::MessageLevel,Msg::NumMessageLevels>;
#endif

//#define NO_VALGEN_STRINGS
#ifndef NO_VALGEN_STRINGS
#include "../src/stringparser.h"
#endif

/** Generic construction, assignment, and copying tests for something
    like an AllocatingPointer or a ValueGenerator */
#define CONSTRUCT_ASSIGN_AND_COPY_TEST(vgtype,type,ch)			\
{  									\
  section(#vgtype "<" #type "> construction and assignment");		\
  typedef vgtype<type> VGType;						\
  type val = ch ## 1;							\
  VGType vv(ch ## 2);							\
  VGType vp(&val);							\
  									\
  FLOAT_MATCH( *vv,      ch ## 2);					\
  FLOAT_MATCH( *vp,      ch ## 1);    VGType vvc(vv);  VGType vpc(vp);	\
  FLOAT_MATCH( *vvc,     *vv);	      					\
  FLOAT_MATCH( *vpc,     *vp);        vv=ch ## 3;  val=ch ## 4;		\
  FLOAT_MATCH( *vv,      ch ## 3);    					\
  FLOAT_MATCH( *vp,      ch ## 4);    *(vv.valueptr())=ch ## 5;		\
  FLOAT_MATCH( *vv,      ch ## 5);    type o=ch ## 6;  vv.relink(&o);  vp.relink(&o);	\
  FLOAT_MATCH( *vv,      ch ## 5);					\
  FLOAT_MATCH( *vp,      ch ## 6);					\
  FLOAT_MATCH( *vvc,     ch ## 2);					\
  FLOAT_MATCH( *vpc,     val);						\
  									\
  section(#vgtype "<" #type "> copy construction");			\
  vvc=vpc;								\
  FLOAT_MATCH( *vvc,     val);        vp=vpc;				\
  FLOAT_MATCH( *vp,      *vpc);       val=ch ## 7;			\
  FLOAT_MATCH( *vp,      val);        vp=vv;				\
  FLOAT_MATCH( *vp,      *vv);        vp=vv;  val=ch ## 8;		\
  FLOAT_MATCH( *vp,      *vv);						\
}



int main(int, char**)
{
  section("Sizes");
#ifdef _CRAYMPP
  EXACT_CHECK( int(sizeof(ValueGenerator<double>)),    24);
  EXACT_CHECK( int(sizeof(AllocatingPointer<double>)), 16);
#else
  EXACT_CHECK( int(sizeof(ValueGenerator<double>)),    12);
  EXACT_CHECK( int(sizeof(AllocatingPointer<double>)), 8);
#endif

  const int i1=1;     const float f1=1.3;     const char* s1="the";         
  const int i2=5;     const float f2=5.8;     const char* s2="quick brown"; 
  const int i3=7;     const float f3=7.2;     const char* s3="fox jumped";  
  const int i4=3;     const float f4=3.4;     const char* s4="over the";    
  const int i5=6;     const float f5=6.6;     const char* s5="lazy";        
  const int i6=1;     const float f6=1.2;     const char* s6="dog";         
  const int i7=4;     const float f7=4.9;     const char* s7="and";         
  const int i8=8;     const float f8=8.8;     const char* s8="died";        

  CONSTRUCT_ASSIGN_AND_COPY_TEST(AllocatingPointer,int,i);
  CONSTRUCT_ASSIGN_AND_COPY_TEST(AllocatingPointer,float,f);
  CONSTRUCT_ASSIGN_AND_COPY_TEST(AllocatingPointer,double,f);
  CONSTRUCT_ASSIGN_AND_COPY_TEST(AllocatingPointer,double,i);
#ifndef NO_VALGEN_STRINGS
  CONSTRUCT_ASSIGN_AND_COPY_TEST(AllocatingPointer,string,s);
#endif
  CONSTRUCT_ASSIGN_AND_COPY_TEST(ValueGenerator,int,i);
  CONSTRUCT_ASSIGN_AND_COPY_TEST(ValueGenerator,double,f);
  
  section("ValueGenerator_Random");
  {
    typedef ValueGenerator_Random<double> RVGType;
    // Semantics for copying differs slightly from regular ValueGenerator
    double rad = 0.0;
    double val = f1;
    RVGType vv(f2);
    RVGType vp(&val,&rad);

    FLOAT_MATCH( *vv,      f2);
    FLOAT_MATCH( *vp,      f1);         RVGType vvc(vv);  RVGType vpc(vp);
    FLOAT_MATCH( *vvc,     *vv);
    FLOAT_MATCH( *vpc,     *vp);        vv=f3;  val=f4;  vv.reset();  vp.reset();
    FLOAT_MATCH( *vv,      f3);
    FLOAT_MATCH( *vp,      f4);         *(vv.valueptr())=f5;
    FLOAT_MATCH( *vv,      f5);         double o=f6; vv.relink(&o); vp.relink(&o);
    FLOAT_MATCH( *vv,      f6);
    FLOAT_MATCH( *vp,      f6);
    FLOAT_MATCH( *vvc,     f2);
    FLOAT_MATCH( *vpc,     f1);

    section("ValueGenerator_Random generated values");
    RVGType v(f2,0.3);

    FLOAT_MATCH( *v,       f2);         v.next();
    RANDOM_MATCH(*v,       f2, 0.3);    v.next();
    RANDOM_MATCH(*v,       f2, 0.3);    v.reset();
    FLOAT_MATCH( *v,       f2);         
  }


  section("ValueGenerator_Correlate");
  {
    typedef ValueGenerator_Correlate<double> RVGType;
    // Semantics for copying differs slightly from regular ValueGenerator
    double unc = 0.0;
    double val = f1;
    //double rad = 0.0;
    RVGType vp(&val,&unc,0,24);
    
    FLOAT_MATCH( *vp,      f1);           RVGType vpc(vp);
    FLOAT_MATCH( *vpc,     *vp);    val=f4;  vp.reset();
    FLOAT_MATCH( *vp,      f4);           double o=f6;  vp.relink(&o);
    FLOAT_MATCH( *vp,      o);
    FLOAT_MATCH( *vpc,     f1);

    section("ValueGenerator_Correlate generated values");
    RVGType v(&val,&unc,0,10);
    unc=0.5;

    FLOAT_MATCH( *v,       val);           v.next();
    RANDOM_MATCH(*v,       val, 10-val);
  }
  
  
  section("ValueGenerator_Increment");
  {
    typedef ValueGenerator_Increment<double> RVGType;
    // Semantics for copying differs slightly from regular ValueGenerator
    double val = f1;
    double rad = 0.0;
    RVGType vv(f2);
    RVGType vp(&val,&rad);
    FLOAT_MATCH( *vv,      f2);
    FLOAT_MATCH( *vp,      f1);
    RVGType vvc(vv);
    RVGType vpc(vp);
    FLOAT_MATCH( *vvc,     *vv);
    FLOAT_MATCH( *vpc,     *vp);
    vv=f3; val=f4;
    vv.reset(); vp.reset();
    FLOAT_MATCH( *vv,      f3);
    FLOAT_MATCH( *vp,      f4);
    *(vv.valueptr())=f5;
    FLOAT_MATCH( *vv,      f5);
    double o=f6;
    vv.relink(&o);
    vp.relink(&o);
    FLOAT_MATCH( *vv,      f6);
    FLOAT_MATCH( *vp,      f6);
    FLOAT_MATCH( *vvc,     f2);
    FLOAT_MATCH( *vpc,     f1);

    section("ValueGenerator_Increment generated values");
    const double init=f2;
    const double inc=0.5;
    RVGType v(init,inc); v.next();
    FLOAT_MATCH( *v,       init+inc);      v.next(); v.next(); v.next();
    FLOAT_MATCH( *v,       init+4*inc);    v.reset();
    FLOAT_MATCH( *v,       init);
  }


  
#ifndef NO_VALGEN_STRINGS
  
#define BASIC_STRINGARGS_TEST(type) { \
      StringArgs rargs(p,"6.2 0.0");   \
      type v(rargs);	                \
      FLOAT_MATCH(*v,6.2);        \
    }

  section("String construction");
  {
    StringParser p;

    ValueGeneratorFactory<double> vgf;
    
    ValueGenerator<>* vv = vgf.create(StringArgs(p,"Random 5.8 0.3"));
    FLOAT_MATCH( **vv,     5.8);         vv->next();
    RANDOM_MATCH(**vv,     5.8, 0.3);
    
    BASIC_STRINGARGS_TEST(ValueGenerator<double>);
    BASIC_STRINGARGS_TEST(ValueGenerator_Random<double>);
    BASIC_STRINGARGS_TEST(ValueGenerator_Increment<double>);
    BASIC_STRINGARGS_TEST(ValueGenerator_Correlate<double>);
  }
#endif

  /** This section is actually intended to test just AllocatingPointer
      using CopyUsingClone; ValueGenerator is just a handy example of
      a class that supports cloning. */
  section("AllocatingPointer<ValueGenerator,CopyUsingClone> construction and assignment");
  {
    typedef ValueGenerator<double> BaseType;
    typedef AllocatingPointer<BaseType, CopyUsingClone<BaseType> > VGType;
    BaseType vg(f1);
    BaseType rad = 0.0;
    BaseType val = f1;
    VGType vv(f2);
    VGType vp(new BaseType(f2),true);

    FLOAT_MATCH( **vv,      f2);
    FLOAT_MATCH( **vp,      f2);
    VGType vvc(vv);
    VGType vpc(vp);
    FLOAT_MATCH( **vvc,     **vv);
    FLOAT_MATCH( **vpc,     **vp);       vv=f3;  val=f4;
    FLOAT_MATCH( **vv,      f3);         *(vv.valueptr())=f5;
    FLOAT_MATCH( **vv,      f5);         BaseType o(f6); vv.relink(&o); vp.relink(&o);
    FLOAT_MATCH( **vv,      f5);
    FLOAT_MATCH( **vp,      f2);
    FLOAT_MATCH( **vvc,     f2);

    section("AllocatingPointer<ValueGenerator,CopyUsingClone> copy construction");
    vvc=vpc;
    vp=vpc;
    FLOAT_MATCH( **vp,      **vpc);      val=f7; vp=vv;
    FLOAT_MATCH( **vp,      **vv);       vp=vv;  val=f8;
    FLOAT_MATCH( **vp,      **vv);
  }

  {
    section("AllocatingPointer<ValueGenerator_Increment,CopyUsingClone>");
    typedef ValueGenerator<int> BaseType;    
    typedef AllocatingPointer<BaseType, CopyUsingClone<BaseType> > VGType;
    typedef ValueGenerator_Increment<int> ActualType;
    ActualType a1(i1);
    VGType vpi(a1);
    EXACT_MATCH( **vpi,      i1);
    vpi->next();
    vpi->next();
    EXACT_MATCH( **vpi,      i1+2);
  }
  
  return count_errors();
}
