# Topographica Project
# August 2003
# Judah De Paula
# Driver program to call DislinPlot with some sample plots.  This way it
# can be tested independently of Topographica proper.
import DislinPlot, math, Image, ImageOps


######  Example From Dislin.  
def generate_zmat(gridsize):
    # Example contour map for simple overlay of bitmap
    n = gridsize
    m = gridsize
    xray = range (n)
    yray = range (m)
    zmat = range (n*m)
    fpi  = 3.1415927 / 180.
    stepx = 360. / (n - 1)
    stepy = 360. / (m - 1)
    
    for i in range (0, n):
      xray[i] = xray[i] * stepx
    
    for i in range (0, m):
      yray[i] = yray[i] * stepy
    
    for i in range (0, n):
      x = xray[i] * fpi
      for j in range (0, m):
        y = yray[j] * fpi
        zmat[i*m+j] = 2 * math.sin(x) * math.sin(y)
    return zmat

def generate_layers(plot):
  for i in range (0, 9):
    zlev = -2. + i * 0.5
    if i == 4:
      plot.add_layer('NONE',zlev,(i+1) * 28)
    else:
      plot.add_layer('FLOAT',zlev,(i+1) * 28)


# First example plot.  Proof of concept, don't use as a template.
def plot1():
    zmat = generate_zmat(50)
    contourplot = DislinPlot.DislinPlot()
    generate_layers(contourplot)
    contourplot.add_grid(50, 50, zmat)
    filename = contourplot.genplot()
    print 'Test plot that uses mostly defaults: ', filename
#######


# Template example for creating a tiff file from DislinPlot
def plot2():
    # Set plot options for a tiff output file
    plot2 = DislinPlot.DislinPlot()
    plot2.set_dislin_scale(False)              # Color will be 256
    plot2.set_bitmap_size(800)                 # Not used for postscript
    plot2.set_display('tiff')
    plot2.set_xname('X Position')
    plot2.set_yname('Y Position')
    plot2.set_input_bmpfile('030423_oo_dir_map_od_512MB.020000.or_Eye1_noselect.bmp')

    # Open up the data file for conversion and put in a list
    image = Image.open('030423_oo_dir_map_od_512MB.020000.od_noselect_bw.bmp')
    imagebw = ImageOps.grayscale(image).rotate(270) # Rotated!??
    (width, height) = image.size
    pixels = list(imagebw.getdata())

    #Add contour, data, and then plot.  Parameter returned is actual filename.
    plot2.add_layer('NONE',255. / 2,255)
    plot2.add_grid(width,height,pixels)
    filename = plot2.genplot()
    print 'Plot 2 file created and named: ', filename


# Template example for creating a postscript file from DislinPlot
def plot3():
    # Set plot options for a postscript output file
    plot2 = DislinPlot.DislinPlot()
    plot2.set_dislin_scale(True)
    #plot2.set_bitmap_size(800)                 # Not used for postscript
    plot2.set_display('postscript')             # Or 'tiff'
    plot2.set_xname('X Position')
    plot2.set_yname('Y Position')
    plot2.set_input_bmpfile('030423_oo_dir_map_od_512MB.020000.or_Eye1_noselect.bmp')

    # Open up the data file for conversion and put in a list
    image = Image.open('030423_oo_dir_map_od_512MB.020000.od_noselect_bw.bmp')
    imagebw = ImageOps.grayscale(image).rotate(270) # Rotated!?? Fortran?
    (width, height) = image.size
    pixels = list(imagebw.getdata())

    #Add contour, data, and then plot.  Parameter returned is actual filename.
    plot2.add_layer('NONE',255. / 2,255)
    plot2.add_grid(width,height,pixels)
    filename = plot2.genplot()
    print 'Plot 3 file created and named: ', filename


# Begin main
plot1()
plot2()
plot3()
